import numpy as np
import datetime
import meteva
from scipy.ndimage.filters import uniform_filter


def mesh_lon_time(grd_ob,grd_fo,method = np.mean,grid = None,interval = None,smooth = 1,cmap = "rainbow",clevs = None,ncol = None,annot =None,dpi = 300,
         spasify_xticks = None,sup_fontsize = 10,title ="",width = None,height = None,save_path = None,show = None):

    if grid is not None:
        grd_ob = meteva.base.interp_gg_linear(grd_ob,grid=grid)
        grd_fo = meteva.base.interp_gg_linear(grd_fo,grid = grid)

    grid_ob = meteva.base.get_grid_of_data(grd_ob)
    time_ob = meteva.base.all_type_time_to_datetime(grid_ob.gtime[0])
    dtimes =grd_fo["dtime"].values
    times_all = grd_fo["time"].values
    times_fo = []
    for time1 in times_all:
        times_fo.append(meteva.base.all_type_time_to_datetime(time1))

    member_list = grd_fo["member"].values

    mean_value_array = np.zeros((len(member_list),len(dtimes)+1,len(grd_ob["lon"].values)))
    time_fo_list = []
    for i in range(len(dtimes)):
        k = len(dtimes) - i - 1
        dtime = dtimes[k]
        time_fo = time_ob -  datetime.timedelta(hours=int(dtime))
        if time_fo in times_fo:
            grd_1dtime =grd_fo.isel(dtime=slice(k, k+1))
            grd_1time = meteva.base.in_time_list(grd_1dtime,time_list=[time_fo])
            mean_value_array[:,i, :] = method(grd_1time.values, axis=4).squeeze()
            time_fo_list.append(time_fo)

    time_fo_list.append(time_ob)
    mean_value_array[:,-1,:] = method(grd_ob.values,axis=4)

    name_list_dict = {
        "member":member_list,
        "时间":time_fo_list,
        "经度":grd_ob["lon"].values
    }

    if interval is not None:
        size = int(interval/grid_ob.dlon)
        size_smooth = smooth * size
        mean_value_array1 = np.zeros(mean_value_array.shape)
        for i in range(mean_value_array.shape[0]):
            for j in range(mean_value_array.shape[1]):
                dat = uniform_filter(mean_value_array[i,j,:], size=size_smooth)
                mean_value_array1[i,j,:] = dat
        mean_value_array = mean_value_array1[:,:,::size]
        name_list_dict["经度"] = name_list_dict["经度"][::size]

    col = mean_value_array.shape[1]
    row = mean_value_array.shape[2]
    meteva.base.mesh(mean_value_array,name_list_dict=name_list_dict,save_path=save_path,cmap=cmap,clevs=clevs,
                     ncol =ncol,rect = [-0.5,col-1.5,row,1],annot = annot,spasify_xticks=spasify_xticks,sup_fontsize=sup_fontsize,title=title
                     ,width=width,height=height,dpi=dpi,show=show)



    pass


def mesh_time_lat(grd_ob,grd_fo,method = np.mean,grid = None,interval = None,smooth = 1,cmap = "rainbow",clevs = None,ncol = None,annot =None,dpi = 300,
         spasify_xticks = None,sup_fontsize = 10,title ="",width = None,height = None,save_path = None,show = None):

    if grid is not None:
        grd_ob = meteva.base.interp_gg_linear(grd_ob,grid=grid)
        grd_fo = meteva.base.interp_gg_linear(grd_fo,grid = grid)

    grid_ob = meteva.base.get_grid_of_data(grd_ob)
    time_ob = meteva.base.all_type_time_to_datetime(grid_ob.gtime[0])
    dtimes =grd_fo["dtime"].values
    times_all = grd_fo["time"].values
    times_fo = []
    for time1 in times_all:
        times_fo.append(meteva.base.all_type_time_to_datetime(time1))

    member_list = grd_fo["member"].values

    mean_value_array = np.zeros((len(member_list),len(dtimes)+1,len(grd_ob["lat"].values)))
    time_fo_list = []
    for i in range(len(dtimes)):
        k = len(dtimes) - i - 1
        dtime = dtimes[k]
        time_fo = time_ob -  datetime.timedelta(hours=int(dtime))
        if time_fo in times_fo:
            grd_1dtime =grd_fo.isel(dtime=slice(k, k+1))
            grd_1time = meteva.base.in_time_list(grd_1dtime,time_list=[time_fo])
            mean_value_array[:,i, :] = method(grd_1time.values, axis=5).squeeze()
            time_fo_list.append(time_fo)

    time_fo_list.append(time_ob)
    mean_value_array[:,-1,:] = method(grd_ob.values,axis=5)

    name_list_dict = {
        "member":member_list,
        "时间":time_fo_list,
        "纬度":grd_ob["lat"].values
    }

    if interval is not None:
        size = int(interval/grid_ob.dlon)
        size_smooth = size * smooth
        mean_value_array1 = np.zeros(mean_value_array.shape)
        for i in range(mean_value_array.shape[0]):
            for j in range(mean_value_array.shape[1]):
                dat = uniform_filter(mean_value_array[i,j,:], size=size_smooth)
                mean_value_array1[i,j,:] = dat
        mean_value_array = mean_value_array1[:,:,::size]
        name_list_dict["纬度"] = name_list_dict["纬度"][::size]

    col = mean_value_array.shape[1]
    row = mean_value_array.shape[2]
    meteva.base.mesh(mean_value_array,name_list_dict=name_list_dict,axis_x="时间",axis_y="纬度",
                     save_path=save_path,cmap=cmap,clevs=clevs,ncol =ncol,rect = [col-1.5,-0.5,1,row+1],
                     annot = annot,spasify_xticks=spasify_xticks,sup_fontsize=sup_fontsize,title=title
                     ,width=width,height=height,dpi=dpi,show=show)

    pass


# -*- coding: utf-8 -*-
"""
Created on Thu Jan 28 17:04:49 2021

@author: 1
"""
import numpy as np

def Earth(x1, x2 = 'NULL', miles = True, R = 'NULL'):
    #计算球面距离
    if (R == 'NULL'):
        if (miles):
            R = 3963.34
        else :
            R = 6378.388
    coslat1 = np.cos((x1[:, 1] * np.pi)/180)
    sinlat1 = np.sin((x1[:, 1] * np.pi)/180)
    coslon1 = np.cos((x1[:, 0] * np.pi)/180)
    sinlon1 = np.sin((x1[:, 0] * np.pi)/180)
    if (x2 == 'NULL'):
        pp1 = np.array((coslat1 * coslon1, coslat1 * sinlon1, sinlat1))
        pp2 = np.array((coslat1 * coslon1, coslat1 * sinlon1, sinlat1))
        pp = np.dot(pp1.T, pp2)
        if float('%.6f'%(np.max(pp))) > 1:
            return R * np.arccos(1 * np.sign(pp))
        else :
            return R * np.arccos(pp)
        
    else :
        coslat2 = np.cos((x2[:, 1] * np.pi)/180)
        sinlat2 = np.sin((x2[:, 1] * np.pi)/180)
        coslon2 = np.cos((x2[:, 0] * np.pi)/180)
        sinlon2 = np.sin((x2[:, 0] * np.pi)/180)
        pp1 = np.mat((coslat1 * coslon1, coslat1 * sinlon1, sinlat1))
        pp2 = np.mat((coslat2 * coslon2, coslat2 * sinlon2, sinlat2))
        pp = np.dot(pp1.T, pp2)
        if float('%.6f'%(np.max(pp))) > 1:
            return R * np.arccos(1 * np.sign(pp))
        else :
            return R * np.arccos(pp)
        
def rdist(Id, loc):
    #计算两点间欧式距离
    dd = []
    for i in range(len(Id)):
        a = Id[i, 0]
        b = Id[i, 1]
        x11 = loc[a]
        x22 = loc[b]
        dd00 = np.sqrt(np.sum(np.square(x11 - x22)))
        dd.append(dd00)
    dd = np.array(dd)
    return dd


#初始化io下边的四个模块，分别为读出格点数据，写入格点数据，读出站点数据，写入站点数据
from .read_griddata import *
from .write_griddata import *
from .read_stadata import *
from .write_stadata import *
from .write_array import *
from .encoding import *
from .read_graphydata import read_micaps14
from .print_grib_info import print_grib_file_info
#from . import DataBlock_pb2
#from .GDS_data_service import GDSDataService


import numpy as np
import math
from . import zapsmall
import copy

def fint2d(x, ws, s, method=None, derivs=False):
    if method is None:
        method = ["nearest", "linear", "cubic"]
    x = np.array(x)
    ws = np.array(ws)
    dimout = x.shape
    if method == "round":
        minx = miny = 1
        maxx = dimout[0]
        maxy = dimout[1]
    elif method == "linear":
        minx = 1
        miny = 1
        maxx = dimout[0] - 1
        maxy = dimout[1] - 1
    elif method == "cubic":
        minx = 2
        miny = 2
        maxx = dimout[0] - 2
        maxy = dimout[1] - 2
    else:
        raise Exception("the input method must be 'nearest', 'linear' or 'cubic'")
    u = ws[:, 0]
    v = ws[:, 1]
    u[u > maxx] = maxx
    u[u < minx] = minx
    v[v > maxy] = maxy
    v[v < miny] = miny
    n = len(u)
    out = np.zeros((dimout[0], dimout[1])) + np.nan
    if derivs:
        out_x = copy.deepcopy(out)
        out_y = copy.deepcopy(out)
        temp_fx = np.array(x[:, 2: dimout[1]] - x[:, 1: (dimout[1] - 1)])
        temp_fxy = np.zeros(temp_fx.size)
        d_fx = cbind(temp_fx, temp_fxy)
        temp_fy = np.array(x[2: dimout[0], :] - x[1: (dimout[0] - 1), :])
        temp_fyy = np.zeros(temp_fy.size)
        d_fy = cbind(temp_fy, temp_fyy.transpose())
    if method == "nearest":
        x = np.floor(u + 0.5)
        x[np.array(x > maxx)] = maxx
        x[np.array(x < minx)] = minx
        y = np.floor(v + 0.5)
        y[np.array(y > maxy)] = maxy
        y[np.array(y < miny)] = miny
        # out[s] = x[cbind(x, y)]
        out[s] = x[cbind(x, y)]
        if derivs:
            out_x[s] = d_fx[cbind(x, y)]
            out_y[s] = d_fy[cbind(x, y)]
            out_x[np.isnan(out_x)] = 0
            out_y[np.isnan(out_y)] = 0
    elif method == "linear":
        index_i = s[:,0].astype(np.int16)
        index_j = s[:,1].astype(np.int16)
        ig = np.floor(ws[:,0]).astype(dtype='int16')
        jg = np.floor(ws[:,1]).astype(dtype='int16')
        dx = ws[:,0] - ig
        dy = ws[:,1] - jg
        c00 = (1 - dx) * (1 - dy)
        c01 = dx * (1 - dy)
        c10 = (1 - dx) * dy
        c11 = dx * dy
        ig1 = ig+1
        jg1 = jg+1
        out = np.zeros(x.shape)
        index_in = np.where((ig>=0)&(ig<maxx-1)&(jg>=0)&(jg<maxy-1))
        index_i = index_i[index_in]
        index_j = index_j[index_in]
        c00 = c00[index_in]
        c01 = c01[index_in]
        c10 = c10[index_in]
        c11 = c11[index_in]
        ig = ig[index_in]
        jg = jg[index_in]
        ig1 = ig1[index_in]
        jg1 = jg1[index_in]


        out[index_i,index_j] = c00 * x[ig, jg] + c10 * x[ig, jg1] + c01 * x[ig1, jg] + c11 * x[ig1, jg1]
        return out

    elif method in ["linear1", "cubic"]:
        fu = np.floor(u + 0.5)
        fu[np.array(fu > maxx)] = maxx
        fu[np.array(fu < minx)] = minx
        fv = np.floor(v + 0.5)
        fv[np.array(fv > maxy)] = maxy
        fv[np.array(fv < miny)] = miny
        ufrac = u - fu
        vfrac = v - fv
        fuv = cbind(fu, fv)
        fuv1 = np.array(fuv)
        fuv1[:, 1] = fuv1[:, 1] + 1
        fu1v = np.array(fuv)
        fu1v[:, 0] = fu1v[:, 0] + 1
        fu1v1 = cbind(fu1v[:, 0], fuv1[:, 1])
        if method == "linear":


            for i in range(s.shape[0]):
                out[(int)(s[i][0]) - 1, (int)(s[i][1]) - 1] = (1 - ufrac[i]) * (1 - vfrac[i]) * x[
                    (int)(fuv[i][0]) - 1, (int)(fuv[i][1]) - 1] + (1 - ufrac[i]) * vfrac[i] * x[
                                                                  (int)(fuv1[i][0]) - 1, (int)(fuv1[i][1]) - 1] + ufrac[
                                                                  i] * (1 - vfrac[i]) * x[
                                                                  (int)(fu1v[i][0]) - 1, (int)(fu1v[i][1]) - 1] + ufrac[
                                                                  i] * vfrac[i] * x[
                                                                  (int)(fu1v1[i][0]) - 1, (int)(fu1v1[i][1]) - 1]
            if derivs:
                for i in range(s.shape[0]):
                    out_x[(int)(s[i][0]) - 1, (int)(s[i][1]) - 1] = (1 - ufrac[i]) * (1 - vfrac[i]) * d_fx[
                        (int)(fuv[i][0]) - 1, (int)(fuv[i][1]) - 1] + (1 - ufrac[i]) * vfrac[i] * d_fx[
                                                                        (int)(fuv1[i][0]) - 1, (int)(fuv1[i][1]) - 1] + \
                                                                    ufrac[i] * (
                                                                            1 - vfrac[i]) * d_fx[
                                                                        (int)(fu1v[i][0]) - 1, (int)(fu1v[i][1]) - 1] + \
                                                                    ufrac[i] * vfrac[i] * d_fx[
                                                                        (int)(fu1v1[i][0] - 1), (int)(fu1v1[i][1]) - 1]
                    out_y[(int)(s[i][0]) - 1, (int)(s[i][1]) - 1] = (1 - ufrac[i]) * (1 - vfrac[i]) * d_fy[
                        (int)(fuv[i][0]) - 1, (int)(fuv[i][1]) - 1] + (1 - ufrac[i]) * vfrac[i] * d_fy[
                                                                        (int)(fuv1[i][0]) - 1, (int)(fuv1[i][1]) - 1] + \
                                                                    ufrac[i] * (
                                                                            1 - vfrac[i]) * d_fy[
                                                                        (int)(fu1v[i][0]) - 1, (int)(fu1v[i][1]) - 1] + \
                                                                    ufrac[i] * vfrac[i] * d_fy[
                                                                        (int)(fu1v1[i][0]) - 1, (int)(fu1v1[i][1]) - 1]
        elif method == "cubic":
            u_bneg1 = (2 * np.power(ufrac, 2) - np.power(ufrac, 3) - ufrac) / 2
            v_bneg1 = (2 * np.power(vfrac, 2) - np.power(vfrac, 3) - vfrac) / 2
            u_b0 = (3 * np.power(ufrac, 3) - 5 * np.power(ufrac, 2) + 2) / 2
            v_b0 = (3 * np.power(vfrac, 3) - 5 * np.power(vfrac, 2) + 2) / 2
            u_b1 = (4 * np.power(ufrac, 2) - 3 * np.power(ufrac, 3) + ufrac) / 2
            v_b1 = (4 * np.power(vfrac, 2) - 3 * np.power(vfrac, 3) + vfrac) / 2
            u_b2 = ((ufrac - 1) * np.power(ufrac, 2)) / 2
            v_b2 = ((vfrac - 1) * np.power(vfrac, 2)) / 2
            fun1vn1 = fuv - 1
            fun1v = cbind(fun1vn1[:, 0], fuv[:, 1])
            fun1v1 = cbind(fun1v[:, 0], fuv[:, 1] + 1)
            fun1v2 = cbind(fun1v[:, 0], fuv[:, 1] + 2)
            fuvn1 = cbind(fuv[:, 0], fun1vn1[:, 1])
            fuv1 = cbind(fuv[:, 0], fun1v1[:, 1])
            fuv2 = cbind(fuv[:, 0], fun1v2[:, 1])
            fu1vn1 = cbind(fuv[:, 0] + 1, fun1vn1[:, 1])
            fu1v = cbind(fu1vn1[:, 0], fuv[:, 1])
            fu1v1 = cbind(fu1vn1[:, 0], fuv1[:, 1])
            fu1v2 = cbind(fu1vn1[:, 0], fun1v2[:, 1])
            fu2vn1 = cbind(fuv[:, 0] + 2, fun1vn1[:, 1])
            fu2v = cbind(fu2vn1[:, 0], fun1v[:, 1])
            fu2v1 = cbind(fu2vn1[:, 0], fun1v1[:, 1])
            fu2v2 = cbind(fu2vn1[:, 0], fun1v2[:, 1])
            if derivs:
                du_bneg1 = (4 * ufrac - 3 * np.power(ufrac, 2) - 1) / 2
                dv_bneg1 = (4 * vfrac - 3 * np.power(vfrac, 2) - 1) / 2
                du_b0 = (9 * np.power(ufrac, 2) - 10 * ufrac) / 2
                dv_b0 = (9 * np.power(vfrac, 2) - 10 * vfrac) / 2
                du_b1 = (8 * ufrac - 9 * np.power(ufrac, 2) + 1) / 2
                dv_b1 = (8 * vfrac - 9 * np.power(vfrac, 2) + 1) / 2
                du_b2 = (3 * np.power(ufrac, 2) - 2 * ufrac) / 2
                dv_b2 = (3 * np.power(vfrac, 2) - 2 * vfrac) / 2
            temp_out = u_bneg1 * (v_bneg1 * get_by_xy(x,fun1vn1) + v_b0 * get_by_xy(x,fun1v) + v_b1 * get_by_xy(x,fun1v1) + v_b2 * get_by_xy(x, fun1v2)) + u_b0 * (v_bneg1 * get_by_xy(x, fuvn1) + v_b0 * get_by_xy(x, fuv) + v_b1 * get_by_xy(x, fuv1) + v_b2 * get_by_xy(x, fuv2)) + u_b1 * (v_bneg1 * get_by_xy(x, fu1vn1) + v_b0 * get_by_xy(x, fu1v) + v_b1 * get_by_xy(x, fu1v1) + v_b2 * get_by_xy(x, fu1v2)) + u_b2 * (v_bneg1 * get_by_xy(x, fu2vn1) + v_b0 * get_by_xy(x, fu2v) + v_b1 * get_by_xy(x, fu2v1) + v_b2 * get_by_xy(x, fu2v2))
            set_by_xy(out, s, temp_out)
            if derivs:
                temp_out_x = du_bneg1 * (v_bneg1 * get_by_xy(d_fx, fun1vn1) + v_b0 * get_by_xy(d_fx, fun1v) + v_b1 * get_by_xy(d_fx, fun1v1) + v_b2 * get_by_xy(d_fx,
                    fun1v2)) + du_b0 * (v_bneg1 * get_by_xy(d_fx, fuvn1) + v_b0 * get_by_xy(d_fx, fuv) + v_b1 * get_by_xy(d_fx, fuv1) + v_b2 * get_by_xy(d_fx,
                    fuv2)) + du_b1 * (v_bneg1 * get_by_xy(d_fx, fu1vn1) + v_b0 * get_by_xy(d_fx, fu1v) + v_b1 * get_by_xy(d_fx, fu1v1) + v_b2 * get_by_xy(d_fx,
                    fu1v2)) + du_b2 * (v_bneg1 * get_by_xy(d_fx, fu2vn1) + v_b0 * get_by_xy(d_fx, fu2v) + v_b1 * get_by_xy(d_fx, fu2v1) + v_b2 * get_by_xy(d_fx,
                    fu2v2))
                set_by_xy(out_x, s, temp_out_x)
                temp_out_y = dv_bneg1 * (u_bneg1 * get_by_xy(d_fy, fun1vn1) + u_b0 * get_by_xy(d_fy, fuvn1) + u_b1 * get_by_xy(d_fy, fu1vn1) + u_b2 * get_by_xy(d_fy,
                    fu2vn1)) + dv_b0 * (u_bneg1 * get_by_xy(d_fy, fun1v) + u_b0 * get_by_xy(d_fy, fuv) + u_b1 * get_by_xy(d_fy, fu1v) + u_b2 * get_by_xy(d_fy,
                    fu2v)) + dv_b1 * (u_bneg1 * get_by_xy(d_fy, fun1v1) + u_b0 * get_by_xy(d_fy, fuv1) + u_b1 * get_by_xy(d_fy, fu1v1) + u_b2 * get_by_xy(d_fy,
                    fu2v1)) + dv_b2 * (u_bneg1 * get_by_xy(d_fy, fun1v2) + u_b0 * get_by_xy(d_fy, fuv2) + u_b1 * get_by_xy(d_fy, fu1v2) + u_b2 * get_by_xy(d_fy,
                    fu2v2))
                set_by_xy(out_y, s, temp_out_y)
    else:
        raise Exception("method must be one of nearest, linear or cubic")
    out = zapsmall.zapsmall(out)
    if derivs:
        return list(xy=out, dx=out_x, dy=out_y)
    else:
        return out


def cbind(x, y):
    return np.vstack((np.array(x), np.array(y))).transpose()

def get_by_xy(matrix, index):
    row_num = int(index.size / 2)
    result = np.zeros(row_num)
    matrix = np.array(matrix)
    index = np.array(index)
    for i in range(row_num):
        if math.isnan(index[i, 0]):
            x = - 1
        else:
            x = int(index[i, 0]) - 1
        if math.isnan(index[i, 1]):
            y = -1
        else:
            y = int(index[i, 1]) - 1
        result[i] = matrix[x, y]
    return result

def set_by_xy(matrix, index, value):
    row_num = int(index.size / 2)
    for i in range(row_num):
        if math.isnan(index[i, 0]):
            x = - 1
        else:
            x = int(index[i, 0]) - 1
        if math.isnan(index[i, 1]):
            y = -1
        else:
            y = int(index[i, 1]) - 1
        matrix[x, y] = value[i]



from meteva.base import IV

def mre_skill(mre, mre_base):
    '''
    mre_skill 相对误差技巧，其中mre是精细化网格预报竞赛检验办法中的水量定量相对误差检验指标，
    :param mre:
    :param mre_base:
    :return: 相对技巧值，取值范围为负无穷到1，但负无穷记为-999返回
    '''
    if mre_base == 0:
        if mre > 0:
            return -999
        else:
            return 0
    else:
        smre1 = (mre_base - mre) / mre_base
        return smre1

def sme(mre, mre_base):
    '''
    :param mre: 省台的定量降水相对误差
    :param mre_base: 中央台的定量降水相对误差
    :return: 省台相对于中央台的技巧
    '''
    if mre_base == 0:
        if mre > 0:
            return -IV
        else:
            return 0
    else:
        skill = (mre_base - mre) / mre_base
        return round(skill,3)

def sst(mae,mae_base):
    '''

    :param mae: 省台的平均绝对误差
    :param mae_base: 中央台的平均绝对误差
    :return: 省台相对于中央台的技巧
    '''


    if mae_base == 0:
        skill = 1.01
    else:
        skill = (mae_base - mae)/(mae_base)

    return round(skill,3)



#from sklearn.metrics import confusion_matrix
import numpy as np
import pandas as pd

def confusion_matrix(x,y):
    d = np.sum(x * y)
    b = np.sum((1-x) * y)
    c = np.sum(x*(1-y))
    a = np.sum((1-x)*(1-y))
    result = np.array([[a,b],[c,d]])
    return result

def contingency_table_yesorno(ob, fo, grade_list=[1e-30],compare =">=",compair = None,member_list=None,  save_path=None):
    '''
    contingency_table 预测列联表
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组
    :param grade_list: grade_list: 多个阈值同时检验时的等级参数
    :param save_path: 保存地址，如果保存地址不为空时会将列联表输出到excel，
    其中每一个sheet为一个等级的列联表
    :return: 返回一个列表，列表中的元素为一个阈值条件下，观测-预报列联表
    '''
    if compair is not None:
        print("warning: the argument compair will be abolished, please use compare instead\n警告：参数compair 将被废除，以后请使用参数compare代替")
        compare = compair
    if compare not in [">=",">","<","<="]:
        print("compare 参数只能是 >=   >  <  <=  中的一种")
        return
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    label = []
    if member_list is None:
        if new_Fo_shape[0] == 1:
            label.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                label.append('预报' + str(i + 1))
    else:
        label.extend(member_list)
    grade_name_list = ['grade=' + str(i) for i in grade_list]
    conf_mx_list = []
    table_data_list = []
    for line in range(new_Fo_shape[0]):

        fo_piece = new_Fo[line, :]

        for grade in grade_list:
            shape = ob.shape
            new_ob = np.zeros(shape)
            new_fo = np.zeros(shape)
            index_list = ["No"]

            if compare == ">=":
                ob_index_list = np.where(ob >= grade)
                fo_index_list = np.where(fo_piece >= grade)
            elif compare == "<=":
                ob_index_list = np.where(ob <= grade)
                fo_index_list = np.where(fo_piece <= grade)
            elif compare == ">":
                ob_index_list = np.where(ob > grade)
                fo_index_list = np.where(fo_piece > grade)
            else:
                ob_index_list = np.where(ob < grade)
                fo_index_list = np.where(fo_piece < grade)

            new_ob[ob_index_list] = 1
            new_fo[fo_index_list] = 1
            index_list.append("Yes")

            new_fo = new_fo.flatten()
            new_ob = new_ob.flatten()
            conf_mx = confusion_matrix(new_fo, new_ob)
            if conf_mx.shape[0] < 2:
                # 说明全都发生或全都不发生
                num = len(new_ob)
                conf_mx = np.zeros((2, 2))
                if new_ob[0] == 0:
                    conf_mx[0, 0] = num
                else:
                    conf_mx[1, 1] = num

            row_sums = conf_mx.sum(axis=1, keepdims=True)
            conf_mx = np.hstack((conf_mx, row_sums))
            line_sums = conf_mx.sum(axis=0, keepdims=True)
            conf_mx = np.vstack((conf_mx, line_sums))
            index_list.append('sum')
            conf_mx_list.append(conf_mx)
            if save_path is not None:
                table_data = pd.DataFrame(conf_mx,
                                          columns=pd.MultiIndex.from_product([['观测'], index_list]),
                                          index=pd.MultiIndex.from_product([['预报'], index_list])
                                          )
                table_data_list.append(table_data)

    if save_path is not None:
        with pd.ExcelWriter(save_path) as writer:
            for i in range(len(table_data_list)):
                a = int(i / len(grade_name_list))
                b = i%len(grade_name_list)

                table_data_list[i].to_excel(writer, sheet_name=label[a]+'_'+grade_name_list[b])
        print("列联表已以excel表格形式保存至" + save_path)
    result = np.array(conf_mx_list)
    if new_Fo_shape[0]>1:
        result = result.reshape(new_Fo_shape[0],len(grade_list),3,3)
    return result

def performance_hfmc(hfmc_array, axis_list_list, suplot_lengend=[1, 0], save_dir=None):
    pass
    '''
    :param ob:
    :param fo:
    :param grade_list:
    :return:

    grade_count = len(grade_list)
    group_count = len(group_list)
    if grade_count == 1:
        #只有一种等级的
        pass
    elif group_list == 1:
        pass
    else:
        pass



    if grade_count == 1 or group_count == 1:
        column = 1
        row = 1
        if grade_count == 1:
            legend_list = group_list
        else:
            legend_list = grade_list
        subplot_count = 1
    else:
        column = 2
        row = int(math.ceil(grade_count/2))
        legend_list = group_list
        subplot_count = grade_count

    plt.figure(figsize=(column * 5, row * 4.5))
    plt.subplots_adjust(wspace = 0.5,hspace = 0.3)

    pod = pod_hfmc(hfmc_array)
    sr = sr_hfmc(hfmc_array)
    x = np.arange(0.0001, 1, 0.0001)
    bias_list = [0.2, 0.4, 0.6, 0.8, 1, 1.25, 1.67, 2.5, 5]
    ts_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]


    for s in range(subplot_count):
        plt.subplot(column, row, s+1)
        for i in range(len(bias_list)):
            bias = bias_list[i]
            y1 = bias * x
            x2 = x[y1 < 1]
            y2 = y1[y1 < 1]
            if bias < 1:
                plt.plot(x2, y2, '--', color='k', linewidth=0.5)
                plt.text(1.01, bias, "bias=" + str(bias))
            elif bias > 1:
                plt.plot(x2, y2, '--', color='k', linewidth=0.5)
                plt.text(1.0 / bias - 0.05, 1.02, "bias=" + str(bias))
            else:
                plt.plot(x2, y2, '-', color='k', linewidth=0.5)

        for i in range(len(ts_list)):
            ts = ts_list[i]
            hf = 1
            x2 = np.arange(ts, 1, 0.001)
            hit = hf * x2
            hfm = hit / ts
            m = hfm - hf
            y2 = hit / (hit + m)
            plt.plot(x2, y2, "--", color="y", linewidth=0.5)
            error = np.abs(y2 - x2)
            index = np.argmin(error)
            sx = x2[index] + 0.02
            sy = y2[index] - 0.02
            plt.text(sx, sy, "ts=" + str(ts))

        colors = cm.get_cmap('rainbow', 128)
        for i in range(sr.shape[0]):
            color_grade = (i +0.5) /len(legend_list)
            plt.plot(sr[i,s], pod[i,s], 'o', color=colors(color_grade),markersize=12,label = legend_list[i])

        plt.xlim(0, 1)
        plt.ylim(0, 1)
        plt.xlabel("成功率",fontsize = 14)
        plt.ylabel("命中率",fontsize = 14)


    if len(legend_list) < 10:
        legend_ncol = len(legend_list)
    else:
        legend_ncol = int(math.ceil(len(legend_list)/2))
    plt.legend(loc='upper center', bbox_to_anchor=(0, -0.05,1,1),ncol = legend_ncol,bbox_transform=plt.gcf().transFigure)
    if save_path is None:
        plt.show()
    else:
        plt.savefig(save_path)
    '''





# 采用以下形式将所有.py 模块导入到rigider 目录下
from . import fint2d
from . import imomenter
from . import mij
from . import  q_loss_rigid
from . import rigid_transform
from . import rigider
from . import zapsmall

# 采用如下形式将5个用户必要的函数导入到mode模块下
from .rigider import rigid_optimal,rigid_simple
from .rigider import plot_value


import numpy as np
import math

def cr(ob,fo,grade_list=[1e-30],compair = ">="):
    '''
    cr指数，代表集合成员集中且于观测一致的程度，即所有成员及观测在某个阈值以上的落区的交集与并集的比值
    :param ob:实况数据 一维的numpy
    :param fo:预测数据 二维的numpy数组
    :param grade_list:多个阈值同时检验时的等级参数
    :return: 一维numpy数组，其中每个元素为0-1的实数，最优值为1
    '''
    #print(fo.shape)
    if compair not in [">=",">","<","<="]:
        print("compair 参数只能是 >=   >  <  <=  中的一种")
        return
    rmse_list = []
    Fo_shape = fo.shape
    Ob_shape = ob.shape

    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return

    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape


    cr_list = []
    grade_num = len(grade_list)
    ensemble_num = new_Fo_shape[0]
    intersecti = np.zeros_like(ob)
    union = np.zeros_like(ob)

    for g in range(grade_num):
        ob1 = np.zeros_like(ob)
        if compair == ">=":
            ob1[ob >=grade_list[g]] = 1
        elif compair == "<=":
            ob1[ob <=grade_list[g]] = 1
        elif compair == ">":
            ob1[ob > grade_list[g]] = 1
        elif compair == "<":
            ob1[ob < grade_list[g]] = 1



        intersecti[:] = ob1[:]
        union[:] = ob1[:]
        for i in range(ensemble_num):
            fo1 = np.zeros_like(ob)
            if compair == ">=":
                fo1[fo[i,:] >= grade_list[g]] = 1
            elif compair == "<=":
                fo1[fo[i, :] <= grade_list[g]] = 1
            elif compair == ">":
                fo1[fo[i, :] > grade_list[g]] = 1
            elif compair == "<":
                fo1[fo[i, :] < grade_list[g]] = 1

            intersecti[:] = intersecti[:] * fo1[:]
            union[:] = union[:] + fo1[:]
        union[union>0] = 1
        union_num = np.sum(union)
        intersecti_num = np.sum(intersecti)
        cr1 = intersecti_num/(union_num + 1e-30)
        cr_list.append(cr1)
    if len(cr_list)>1:
        crs = np.array(cr_list)
    else:
        crs = cr_list[0]
    return crs


def crps(Ob, Fo):
    '''

    :param Ob:
    :param Fo:
    :return:
    '''
    n = Ob.size
    shape0 = Fo.shape
    m = shape0[0]
    ob0 = Ob.flatten()
    fo0 = Fo.reshape((m, n))
    fo0 = np.sort(fo0)
    minv = min(np.min(ob0), np.min(fo0))
    maxv = max(np.max(ob0), np.max(fo0))

    fo1 = np.zeros((m + 2, n))
    fo1[0, :] = minv
    fo1[1:-1, :] = fo0[:, :]
    fo1[-1, :] = maxv

    crps0 = np.zeros(n)
    for i in range(1, m + 1):
        a = np.zeros(n)
        index1 = np.where((fo1[i, :] < ob0) & (fo1[i + 1] >= ob0))
        a[index1] = ob0[index1] - fo1[i, index1]

        index2 = np.where(fo1[i + 1] < ob0)
        a[index2] = fo1[i + 1, index2] - fo1[i, index2]

        crps0 += a * ((i / m) * (i / m))

    for i in range(0, m):
        b = np.zeros(n)
        index1 = np.where(ob0 <= fo1[i])
        b[index1] = fo1[i + 1, index1] - fo1[i, index1]

        index2 = np.where((fo1[i, :] < ob0) & (fo1[i + 1] >= ob0))
        b[index2] = fo1[i + 1, index2] - ob0[index2]

        crps0 += b * ((1 - i / m) * (1 - i / m))

    crps_mean = np.mean(crps0)
    return crps_mean



def variance_mse(ob,fo):
    '''
    :param ob:实况数据 一维的numpy
    :param fo:预测数据 二维的numpy数组
    :return:
    '''
    shape0 = fo.shape
    m = shape0[0]
    mean_fo = np.mean(fo,axis=0)
    mse = np.mean(np.square(mean_fo - ob))
    variance = 0
    for i in range(m):
        variance += np.mean(np.square((fo[i,...] - mean_fo)))

    variance /=m
    variance *= (m+1)/m  #膨胀因子

    return variance,mse

def variance_divide_by_mse(ob,fo):
    '''

    :param ob:实况数据 一维的numpy
    :param fo:预测数据 二维的numpy数组
    :return: 平均集合内方差/
    '''
    variance,mse = variance_mse(ob,fo)
    return variance/mse


def std_rmse(ob,fo):
    '''

    :param ob:实况数据 一维的numpy
    :param fo:预测数据 二维的numpy数组
    :return: 平均集合内方差/
    '''
    variance,mse = variance_mse(ob,fo)
    return math.sqrt(variance),math.sqrt(mse)







import numpy as np
import math
import pandas as pd
import copy

def surround_index(grd_ob,grd_fo,thresholds):
    '''

    :param grd_ob: 观测网格数据
    :param grd_fo: 预报网格数据
    :param threholds: 等级阈值
    :return:
    '''
    x = copy.deepcopy(grd_ob.values.squeeze())
    aindex_x = sindex(x,thresh=thresholds)
    y = copy.deepcopy(grd_fo.values.squeeze())
    aindex_y = sindex(y, thresh=thresholds)
    resulut = {"surround_index":[aindex_x["surround_index"],aindex_y["surround_index"]],
               "perimeter_min":[aindex_x["perimeter_min"],aindex_y["perimeter_min"]],
               "perimeter":[aindex_x["perimeter"],aindex_y["perimeter"]]}
    return resulut



def sindex(x, thresh=None, loc=None):

    if thresh is None:
        thresh = 1e-08
    t = copy.deepcopy(x)
    t[t <= thresh] = 0       #低于阈值的置为0
    n = np.count_nonzero(t)   #非0的格点数
    n2 = math.sqrt(n)
    if math.floor(n2) == n2:
        #正方形网格
        p_min = 4 * n2  #周长
    else:
        #长方形网格， 例如 n=4*5 = 20, n2 = 4.47, math.floor(2 * n2)=8,p_min = 18
        p_min = 2 * (math.floor(2 * n2) + 1)   #周长

    if loc is None:
        x_dim = x.shape[0]
        y_dim = x.shape[1]
        range0 = np.tile(np.arange(1, x_dim + 1), y_dim)
        range1 = (np.arange(1, y_dim + 1)).repeat(x_dim)
        loc = np.stack((range0, range1), axis=-1)

    index = t != 0
    i = np.tile(index.reshape(index.size, 1), 2)
    ft = loc[i]
    ft = ft.reshape(((int)(ft.size/2), 2))
    ft = np.array(np.where(t>0)).T
    ft_max = np.max(ft, axis=0)
    ft_min = np.min(ft, axis=0)
    p = 2 * (ft_max[0] - ft_min[0] + 1) + 2 * (ft_max[1] - ft_min[1] + 1)
    #res = pd.DataFrame({"surround_index": p_min / p, "perimeter_min": p_min, "perimeter": p}, index=[0])
    res = {"surround_index": p_min / p, "perimeter_min": p_min, "perimeter": p}
    return res


if __name__ == '__main__':
    # x = np.zeros((8, 8))
    # x[2, 1:4] = 2
    # x[4, 3:6] = 1
    # x[6, 5:7] = 1
    # res = sindex(x)
    # print(res)
    import meteva.base as meb
    import meteva.method as mem
    grid2 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    dat_ob2 = np.zeros((grid2.nlat, grid2.nlon))
    for j in range(grid2.nlat):
        for i in range(grid2.nlon):
            dat_ob2[j, i] = 20 * math.exp(-0.001 * (i - 200) ** 2 - 0.001 * (j - 200) ** 2)
    grd_ob2 = meb.grid_data(grid2, dat_ob2)

    grid2 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    dat_fo2 = np.zeros((grid2.nlat, grid2.nlon))
    for j in range(grid2.nlat):
        for i in range(grid2.nlon):
            dat_fo2[j, i] = 10 * math.exp(-0.001 * (i - 230) ** 2 - 0.0003 * (j - 230) ** 2)
            dat_fo2[j, i] += 10 * math.exp(-0.001 * (i - 170) ** 2 - 0.0003 * (j - 170) ** 2)
    grd_fo2 = meb.grid_data(grid2, dat_fo2)
    result = mem.space.surround_index(grd_ob2, grd_fo2, thresholds=5)
    print(result)


import numpy as np
import pandas


def fuzzyjoint2dfun(s_py, s_px, subset=None):
    out = {}
    if type(s_px) == pandas.DataFrame:
        s_px = s_px.fillna(0)
    else:
        s_px = pandas.DataFrame(s_px).fillna(0)
    if type(s_py) == pandas.DataFrame:
        s_py = s_py.fillna(0)
    else:
        s_py = pandas.DataFrame(s_py).fillna(0)

    def vxfun(n11, n01, n10, n00):
        pod = n11 / (n11 + n01)
        far = n10 / (n11 + n10)
        hits_random = (n11 + n01) * (n11 + n10) / (n11 + n01 + n10 + n00)
        ets = (n11 - hits_random) / (n11 + n01 + n10 - hits_random)
        return {'pod': pod, 'far': far, 'ets': ets}

    if subset is None:
        # https://stackoverflow.com/questions/35517220/finding-minima-and-replacing-values-in-pandas-dataframe
        hits = sum(np.sum(np.minimum(s_px, s_py)))  #命中等于x和y中的最小值之和
        miss = sum(np.sum(np.minimum(s_px, 1 - s_py)))  # 漏报等于x 和1-y的最小值 之和
        fa = sum(np.sum(np.minimum(1 - s_px, s_py)))
        cn = sum(np.sum(np.minimum(1 - s_px, 1 - s_py)))
        out["fuzzy"] = vxfun(hits, miss, fa, cn)
        hits = sum(np.sum(s_px * s_py))
        miss = sum(np.sum(s_px * (1 - s_py)))
        fa = sum(np.sum((1 - s_px) * s_py))
        cn = sum(np.sum((1 - s_px) * (1 - s_py)))
        out["joint"] = vxfun(hits, miss, fa, cn)
    else:
        hits = sum(s_px[subset] < s_py[subset])
        miss = sum(s_px[subset] < (1 - s_py[subset]))
        fa = sum((1 - s_px[subset]) < s_py[subset])
        cn = sum((1 - s_px[subset]) < (1 - s_py[subset]))
        out["fuzzy"] = vxfun(hits, miss, fa, cn)
        hits = sum((s_px[subset]) * (s_py[subset]))
        miss = sum((s_px[subset]) * (1 - s_py[subset]))
        fa = sum((1 - s_px[subset]) * s_py[subset])
        cn = sum((1 - s_px[subset]) * (1 - s_py[subset]))
        out["joint"] = vxfun(hits, miss, fa, cn)
    return out



# -*- coding: utf-8 -*-
"""
Created on Fri Jul 24 16:11:44 2020

@author: 1
"""

#============================  deltamm()  ==============================
#Function:使用增量度量方法合并或匹配两个字段内已识别特征

import time
import copy
import sys
from . import deltammSqCen



def deltamm(look_ff, p = 2, show = False):
    const = float("Inf")
    max_delta = float("Inf")
    fun_type = "sqcen"
    N = 701
    x = copy.deepcopy(look_ff)
    if show:
        begin_tiid = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))

    if (x['grd_ob_features'] == None and x['grd_fo_features'] == None):
        try:
            sys.exit(0)
        except:
            print("deltamm: no features to merge/match!")
         
    if (x['grd_ob_features'] == None):
        try:
            sys.exit(0)
        except:
            print("deltamm: no verification features present.")
    if (x['grd_fo_features'] == None):
        try:
            sys.exit(0)
        except:
            print("deltamm: no model features present.")
    out = x.copy()
    #向字典里追加元素
    out.update ({"match_type": "deltamm", "match_message":"Objects merged/matched based on the Baddeley Delta metric via the deltamm function." })
    #a <- attributes(x)    #将x包含的属性信息赋值给a
    #type <- match.arg(type)
    #未翻译
    if (fun_type == "original"):
        print ("提示：该函数暂未翻译，请执行sqcen函数。")
        #res <- deltammOrig(x = x, p = p, max.delta = max.delta, const = const, show = show, ...)
    if (fun_type == "sqcen"):
        res = deltammSqCen.deltammSqcen(x = x, p = p, max_delta = max_delta, const = const, N = N, show = show)
    out.update({'Q': res['Q']})
    out.update({'unmatched':res['unmatched'], 'matches':res['matches'], 'ob_matches':res['Xmatches_number'],
                'merges':res['merges'], 'MergeForce':True, 'class_out':'unmatched'})
    return out

#=============================  Example  ===================================
'''
hold = make_SpatialVx_PA1.makeSpatialVx(X = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\pert000.csv"), 
                    Xhat = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\pert004.csv"), 
                    loc = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\ICPg240Locs.csv"), 
                    thresholds = [0.01, 20.01], projection = True, subset = None, timevals = None, reggrid = True,
                    Map = True, locbyrow = True, fieldtype = "Precipitation", units = ("mm/h"), dataname = "ICP Perturbed Cases", obsname = "pert000", 
                    modelname = "pert004" , q = (0, 0.1, 0.25, 0.33, 0.5, 0.66, 0.75, 0.9, 0.95), qs = None)

look_FeatureFinder = FeatureFinder_test_PA3.featureFinder(Object = hold, smoothfun = "disk2dsmooth", 
                     dosmooth = True, smoothpar = 17, smoothfunargs = None,
                     thresh = 310, idfun = "disjointer", minsize = np.array([1]),
                     maxsize = float("Inf"), fac = 1, zerodown = False, timepoint = 1,
                     obs = 1, model = 1)

#deltamm参数
x = look.copy()
p = 2
max_delta = float("Inf")
const = float("Inf")
fun_type = "sqcen"
N = 701
show = False    #R:是否需要打印在屏幕
look_deltamm = deltamm(x = look_FeatureFinder.copy(), p = 2, max_delta = float("Inf"), const = float("Inf"), fun_type = "sqcen", N = 701, show = False )
'''



















#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Dec 13 17:50:26 2021

"""
import numpy as np
import pandas as pd
import meteva.method as mem
from sklearn.utils import resample


def LocSig(Z, numrep=1000, block_length='NULL', bootfun="mean",
           alpha=0.05, bca=False):
    booted = {}
    if bootfun == 'mean':
        Z = np.ma.masked_array(Z, np.isnan(Z))
        bootfun = np.mean(Z, axis=0)
        booted_t0 = bootfun
    Z = np.array(Z)
    zdim = Z.shape
    n = zdim[0]
    m = zdim[1]
    out = pd.DataFrame(np.zeros(shape=(m, 3)), columns=['Lower', 'Estimate', 'Upper'])
    if block_length is 'NULL':
        block_length = np.floor(np.sqrt(n))
    if block_length == 1:
        # 判断为假，不执行
        # booted = boot(Z, bootfun, R = numrep)    #未翻译
        for i in range(Z.shape[1]):
            booted_list = [np.mean(resample(Z[:, i], n_samples=Z.shape[0])) for _ in
                           range(numrep)]  # n_samples是受tseries控制
            booted_0 = np.array(booted_list)
            booted = np.append(booted, booted_0)
    else:
        # booted = tsboot(Z, bootfun, l = block.length, R = numrep, sim = "fixed")
        booted = np.array(())
        for i in range(Z.shape[1]):
            booted_list = [np.mean(resample(Z[:, i], n_samples=Z.shape[0])) for _ in
                           range(numrep)]  # n_samples是受tseries控制
            booted_0 = np.array(booted_list)
            booted = np.append(booted, booted_0)
            # booted = np.stack((booted, booted_0), axis = 1)
    booted_t = booted.reshape(numrep, Z.shape[1], order='F')
    booted = {'t0': booted_t0, 't': booted_t, 'R': numrep}
    out['Estimate'] = booted['t0']
    #print(booted_t)
    if block_length == 1 and bca:
        # 判断为假,不执行
        conf = 1 - alpha
        tmp_Lower = np.percentile(a=booted_t, q=100*(1 + conf) / 2, axis=0)
        tmp_Upper = np.percentile(a=booted_t, q=100*(1 - conf) / 2, axis=0)
        out['Lower'] = tmp_Lower
        out['Upper'] = tmp_Upper

        '''
        #原本R的逻辑
        for i in range(1, m):
            tmp = boot_CI(booted, conf = 1-alpha, Type = "bac", index = i)
            out['Lower'][i] = tmp["bca"][:, 4]
            out['Upper'][i] = tmp["bca"][:, 5]
        '''
    else:
        if bca:
            print(
                "LocSig: You chose to use the BCa method, but block.length != 1.  Using percentile method with circular block bootstrap instead.")

        conf = 1 - alpha
        tmp_Lower = np.percentile(a=booted_t, q=100*(1 - conf) / 2, axis=0)
        tmp_Upper = np.percentile(a=booted_t, q=100*(1 + conf) / 2, axis=0)
        out['Lower'] = tmp_Lower
        out['Upper'] = tmp_Upper

        '''
        #原本R的逻辑
        for i in range(1, m):
            #计算置信度区间（分位数:np.percentile）
            #可信度=0.90,置信区间宽度=0.05, 置信区间上限=0.07,置信区间下限=0.02,表示:发生在区间(0.02,0.07)这个范围内的可能性为0.90
            tmp = boot_CI(booted, conf = 1-alpha, Type = "perc", index = i)
            out['Lower'[i]] = tmp["prec"][:, 4]
            out['Upper'][i] = tmp["prec"][:, 5]
        '''

    # out['class'] = "LocSig"
    return out


'''    
if __name__ == '__main__':
    obsData = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/GFSNAMobsEx.rda')['GFSNAMobsEx']
    fcstData = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/GFSNAMfcstEx.rda')['GFSNAMfcstEx']
    locData = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/GFSNAMlocEx.rda')['GFSNAMlocEx']
    locId = (locData["Lon"] >= -90) & (locData["Lon"] <= -75) & (locData["Lat"] <= 40)
    locSub = locData[locId]
    GFSobsSub = obsData.T[locId].T
    GFSfcstSub = fcstData.T[locId].T

    Z = GFSobsSub - GFSfcstSub
    numrep = 500
    block_length = 'NULL'
    alpha = 0.05
'''




import meteva

def error_scatter(sta_ob_and_fos,method,s = None,g = None,gll = None,group_name_list= None,save_dir=None,save_path = None,show = False,
                 print_max = 0,print_min = 0,threshold = 0,add_county_line = False,map_extend = None,dpi = 300,title=None,
                  sup_fontsize=10,
                  height=None, width=None
                  ):
    if len(sta_ob_and_fos.index) == 0:
        print("error infomation: 站点数据内容为空")
        return

    if s is not None:
        if g is not None:
            if g == "last_range" or g == "last_step":
                s["drop_last"] = False
            else:
                s["drop_last"] = True
    if g == "id":
        print("绘制误差平面分布时，g 不能设置为id")
        return
    sta_ob_and_fos1 = meteva.base.sele_by_dict(sta_ob_and_fos,s = s)
    if(len(sta_ob_and_fos1.index) == 0):
        print("there is no data to verify")
        return
    sta_ob_and_fos_list, gll1 = meteva.base.group(sta_ob_and_fos1, g = g, gll = gll)
    g_num = len(sta_ob_and_fos_list)
    #if(g_num == 1):gll1 = [None]
    if group_name_list is None:
        group_name_list = meteva.product.program.get_group_name(gll1)

    data_names = meteva.base.get_stadata_names(sta_ob_and_fos_list[0])
    fo_num = len(data_names) - 1

    if isinstance(title, list):
        if fo_num * g_num != len(title):
            print("手动设置的title数目和要绘制的图形数目不一致")
            return

    if save_path is not None:
        if isinstance(save_path, str):
            save_path = [save_path]
        if fo_num * g_num != len(save_path):
            print("手动设置的save_path数目和要绘制的图形数目不一致")
            return

    result_all = []

    for k in range(g_num):
        sta_ob_and_fos_g1 = sta_ob_and_fos_list[k]
        rmse_sta = meteva.product.score_id(sta_ob_and_fos_g1,method,plot = None)[0]

        if isinstance(title, list):
            title1 = title[k * fo_num: (k + 1) * fo_num]
        else:
            title1 = []
            for i in range(fo_num):
                title1.append(meteva.product.program.get_title_from_dict(title, s, g, group_name_list[k], data_names[i+1]))

        save_path1 = None
        if save_path is None:
            if save_dir is None:
                show = True
            else:
                save_path1 = []
                for i in range(len(title1)):
                    fileName = title1[i].replace("\n", "").replace(":", "")
                    save_path1.append(save_dir + "/" + fileName + ".png")
        else:
            save_path1 = save_path[k * fo_num: (k + 1) * fo_num]


        meteva.base.tool.plot_tools.scatter_sta(rmse_sta, save_path=save_path1, show=show,
                                                fix_size=False, title=title1, print_max=print_max,print_min = print_min
                                                , add_county_line=add_county_line,
                                                threshold=threshold, map_extend=map_extend, dpi=dpi,sup_fontsize = sup_fontsize,
                                                width = width,height=height)
        result_all.append(rmse_sta)


    return result_all,gll1

def rmse_scatter(sta_ob_and_fos,s = None,g = None,gll = None,group_name_list= None,save_dir=None,save_path = None,show = False,
                 print_max = 1,threshold = 0,add_county_line = False,map_extend = None,dpi = 300,title="均方根误差站点分布",
                 sup_fontsize=10,
                 height=None, width=None
                 ):

    return error_scatter(sta_ob_and_fos,meteva.method.rmse,s = s,g = g,gll = gll,group_name_list= group_name_list,
                  save_dir= save_dir,save_path = save_path,show = show,print_max =print_max,threshold = threshold,
                  add_county_line= add_county_line,map_extend=map_extend,dpi = dpi,title = title,
                  sup_fontsize=sup_fontsize,
                  height=height, width=width
                  )

def mae_scatter(sta_ob_and_fos,s = None,g = None,gll = None,group_name_list= None, save_dir=None,save_path = None,show = False,
                print_max = 1,threshold = 0,add_county_line = False,map_extend= None,dpi = 300,title="绝对误差站点分布图",
                sup_fontsize=10,
                height=None, width=None
                ):

    return error_scatter(sta_ob_and_fos,meteva.method.mae,s = s,g = g,gll = gll,group_name_list= group_name_list,
                  save_dir= save_dir,save_path = save_path,show = show,print_max =print_max,threshold = threshold,
                  add_county_line= add_county_line,map_extend=map_extend,dpi = dpi,title = title,
                  sup_fontsize=sup_fontsize,
                  height=height, width=width
                  )



def me_scatter(sta_ob_and_fos,s= None,g = None,gll = None,group_name_list= None, save_dir=None,save_path = None,show = False,
               print_max = 1,print_min = 1,threshold = 0,add_county_line = False,map_extend= None,dpi = 300,title="误差站点分布图",
               sup_fontsize=10,
               height=None, width=None
               ):

    return error_scatter(sta_ob_and_fos,meteva.method.me,s = s,g = g,gll = gll,group_name_list= group_name_list,
                  save_dir= save_dir,save_path = save_path,show = show,print_max =print_max,print_min = print_min,
                  threshold = threshold,add_county_line= add_county_line,map_extend=map_extend,dpi = dpi,title = title,
                  sup_fontsize=sup_fontsize,
                  height=height, width=width
                  )



import meteva
import math
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import datetime


def process_compare(ob_list ,fo_list_list ,map_extend = None ,width = 12 ,dpi = 300 ,sup_title = None ,sup_fontsize = 10,
                    add_county_line = False ,add_worldmap = True, xticks_inter = None ,yticks_inter = None ,grid = True,
                    linewidth = [0.3 ,0.3 ,0.2] ,color = ["k" ,"k" ,"k"] ,cmap ="rainbow" ,clevs= None ,save_path = None):
    nrow = len(fo_list_list) + 1
    ncol = len(ob_list)
    for i in range(len(fo_list_list)):
        if ncol < len(fo_list_list[i]):
            ncol = len(fo_list_list[i])

    if map_extend is None:
        map_extend = meteva.base.get_grid_of_data(ob_list[0])


    if isinstance(map_extend, list):
        slon = map_extend[0]
        elon = map_extend[1]
        slat = map_extend[2]
        elat = map_extend[3]
        rlon = elon - slon
        rlat = elat - slat
    else:
        slon = map_extend.slon
        slat = map_extend.slat
        elon = map_extend.elon
        elat = map_extend.elat
        rlon = map_extend.elon - map_extend.slon
        rlat = map_extend.elat - map_extend.slat

    if sup_title is None:
        sup_height_title = 0
    else:
        sup_height_title = sup_fontsize * 0.12

    height_title = sup_fontsize * 0.1
    height_bottem_xticsk = sup_fontsize * 0.05
    height_hspace = 0
    width_wspace = 0
    width_colorbar = 0.1
    width_left_yticks = sup_fontsize * 0.2

    width_all_plot = width - width_colorbar - width_left_yticks - (ncol - 1) * width_wspace
    width_map = width_all_plot / ncol
    height_map = width_map * rlat / rlon
    height_all_plot = height_map * nrow + (nrow - 1) * height_hspace
    height = height_all_plot + height_title + height_bottem_xticsk + sup_height_title

    vmax = elon
    vmin = slon
    if xticks_inter is None:
        r = rlon
        if r <= 0.1:
            inte = 0.05
        elif r <= 0.5:
            inte = 0.1
        elif r <= 1:
            inte = 0.2
        elif r <= 5 and r > 1:
            inte = 1
        elif r <= 10 and r > 5:
            inte = 2
        elif r < 20 and r >= 10:
            inte = 4
        elif r <= 30 and r >= 20:
            inte = 5
        elif r < 180:
            inte = 10
        else:
            inte = 20
    else:
        inte = xticks_inter

    vmin = inte * (math.ceil(vmin / inte))
    vmax = inte * ((int)(vmax / inte ) +0.5)

    xticks = np.arange(vmin, vmax, inte)
    xticks_label = []
    xticks_label_None = []
    for x in range(len(xticks)):
        v1 = xticks[x]
        if abs(v1 - int(v1) ) <1e-7:
            xticks_label.append(str(int(round(v1, 6))))
        else:
            xticks_label.append(str(round(v1, 6)))
        xticks_label_None.append("")
    if xticks[-1] > 0:
        xticks_label[-1] = "   " + xticks_label[-1] + "°E"
    else:
        xticks_label[-1] = "   " + xticks_label[-1] + "°W"

    vmax = elat
    vmin = slat
    if yticks_inter is None:
        r = rlat
        if r <= 0.05:
            inte = 0.01
        elif r <= 0.3:
            inte = 0.05
        elif r <= 1:
            inte = 0.1
        elif r <= 5 and r > 1:
            inte = 1
        elif r <= 10 and r > 5:
            inte = 2
        elif r < 20 and r >= 10:
            inte = 4
        elif r <= 30 and r >= 20:
            inte = 5
        else:
            inte = 10
    else:
        inter = yticks_inter

    vmin = inte * (math.ceil(vmin / inte))
    vmax = inte * ((int)(vmax / inte ) +0.5)
    yticks = np.arange(vmin, vmax, inte)
    yticks_label = []
    yticks_label_None = []
    for y in range(len(yticks)):
        v1 = yticks[y]
        if abs(v1 - int(v1) ) <1e-7:
            v1 = int(round(v1, 6))
        else:
            v1 = round(v1, 6)

        if yticks[y] >= 0:
            yticks_label.append(str(v1) + "°N")
        else:
            yticks_label.append(str(v1) + "°S")
        yticks_label_None.append("")
    fig = plt.figure(figsize=(width, height), dpi=dpi)

    y_sup_title = (height_bottem_xticsk  + nrow * height_map +(nrow - 1) * height_hspace) / height
    if sup_title is not None:
        plt.suptitle(sup_title, x=width_left_yticks / width, y=y_sup_title, fontsize=sup_fontsize,
                     horizontalalignment="left",
                     verticalalignment="top")

    valid_time_list = []
    print(len(ob_list))
    for i in range(len(ob_list)):
        print(ob_list[i]["time"].values)
        time1 = meteva.base.all_type_time_to_datetime(ob_list[i]["time"].values[0])
        valid_time_list.append(time1)

    for j in range(len(fo_list_list)):
        fo_list = fo_list_list[j]
        for i in range(len(fo_list)):
            dh = int(fo_list[i]["dtime"].values[0])
            time0 = meteva.base.all_type_time_to_datetime(fo_list[i]["time"].values[0])
            time1 = time0 + datetime.timedelta(hours=dh)
            valid_time_list.append(time1)
    valid_time_list = list(set(valid_time_list))
    valid_time_list.sort()

    col_dict = {}
    for i in range(len(valid_time_list)):
        col_dict[valid_time_list[i]] = i

    #     cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,cut_colorbar = False)
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N-1)
    #     im = ax.contourf(x, y, np.squeeze(grd1.values), levels=clevs1, cmap=cmap1,norm = norm)

    # 绘制观测
    ax_list_list = []
    ax_list = []
    for i in range(len(ob_list)):
        time1 = meteva.base.all_type_time_to_datetime(ob_list[i]["time"].values[0])
        pi = col_dict[time1]
        pj = 0

        rect1 = [(width_left_yticks + pi * (width_map + width_wspace)) / width,
                 (height_bottem_xticsk + (nrow - 1 - pj) * (height_map + height_hspace)) / height,
                 width_map / width,
                 height_map / height]

        ax = plt.axes(rect1)
        ax.set_xlim((slon, elon))
        ax.set_ylim((slat, elat))

        if i == 0:
            ax.set_yticks(yticks)
            ax.set_yticklabels(yticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
        else:
            ax.set_yticks(yticks)
            ax.set_yticklabels("", fontsize=sup_fontsize * 0.9) #, family='Times New Roman')

        ax.set_xticks(xticks)
        ax.set_xticklabels("", fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
        im = meteva.base.add_contourf(ax, ob_list[i], add_colorbar=False, cmap=cmap, cut_colorbar=False)
        if grid: plt.grid(linestyle="--", linewidth=0.5)
        meteva.base.tool.plot_tools_adv.add_map(ax, add_county_line=add_county_line, add_worldmap=add_worldmap,
                                        sup_fontsize=sup_fontsize, title=None,
                                        linewidth=linewidth, color=color)

        tex = meteva.base.get_path(r"DD日HH时", time1)
        ix = slon + 0.02 * (elon - slon) * 5 / width_map
        iy = elat - 0.035 * (elat - slat) * 5 / height_map
        plt.text(ix, iy, tex, bbox=dict(fc='white', ec='white', pad=0), fontsize=sup_fontsize, zorder=100)
        ax_list.append(ax)

        if i == 0:
            location = [(width_left_yticks + ncol * (width_map + width_wspace) + 0.02) / width,
                        height_bottem_xticsk / height,
                        (width_colorbar - 0.02) / width,
                        height_map * nrow / height]

            colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
            plt.colorbar(im, cax=colorbar_position)

    ax_list_list.append(ax_list)

    for j in range(len(fo_list_list)):
        fo_list = fo_list_list[j]
        ax_list = []
        for i in range(len(fo_list)):
            dh = int(fo_list[i]["dtime"].values[0])
            time0 = meteva.base.all_type_time_to_datetime(fo_list[i]["time"].values[0])
            time1 = time0 + datetime.timedelta(hours=dh)
            pi = col_dict[time1]
            pj = j + 1

            rect1 = [(width_left_yticks + pi * (width_map + width_wspace)) / width,
                     (height_bottem_xticsk + (nrow - 1 - pj) * (height_map + height_hspace)) / height,
                     width_map / width,
                     height_map / height]

            ax = plt.axes(rect1)

            ax.set_xlim((slon, elon))
            ax.set_ylim((slat, elat))
            meteva.base.add_contourf(ax, fo_list[i], add_colorbar=False, cmap=cmap)
            if i == 0:
                ylable = meteva.base.get_path("DD\n日\nHH\n时", time0)
                ax.set_ylabel(ylable, fontsize=sup_fontsize * 0.9, rotation="horizontal", verticalalignment="center")
                ax.set_yticks(yticks)
                ax.set_yticklabels(yticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
            else:
                ax.set_yticks(yticks)
                ax.set_yticklabels("", fontsize=sup_fontsize * 0.9) #, family='Times New Roman')

            if pj == len(fo_list_list) and pi == ncol - 1:
                ax.set_xticks(xticks)
                ax.set_xticklabels(xticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
            elif pj == len(fo_list_list):
                ax.set_xticks(xticks[:-1])
                ax.set_xticklabels(xticks_label[:-1], fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
            else:
                ax.set_xticks(xticks)
                ax.set_xticklabels("", fontsize=sup_fontsize * 0.9) #, family='Times New Roman')

            if grid: plt.grid(linestyle="--", linewidth=0.5)
            meteva.base.tool.plot_tools_adv.add_map(ax, add_county_line=add_county_line, add_worldmap=add_worldmap,
                                            sup_fontsize=sup_fontsize, title=None,
                                            linewidth=linewidth, color=color)

            tex = str(dh).zfill(3) + "H"
            ix = slon + 0.02 * (elon - slon) * 5 / width_map
            iy = elat - 0.035 * (elat - slat) * 5 / height_map
            plt.text(ix, iy, tex, bbox=dict(fc='white', ec='white', pad=0), fontsize=sup_fontsize, zorder=100)
            ax_list.append(ax)

    ax_list_list.append(ax_list)


# -*- coding: utf-8 -*-
"""
Created on Fri Jan 22 17:52:38 2021

@author: 1
"""
import numpy as np
import meteva
import matplotlib.pyplot as plt
import math


def structurogram_matrix(grd_list, delta = 10, step = 1, q = 1):


    if not isinstance(grd_list,list):
        grd_list = meteva.base.split_grd(grd_list,used_coords=["member"])
    if not isinstance(grd_list,list):
        grd_list = [grd_list]

    names = []
    for grd in grd_list:
        names.append(grd['member'].values[0])

    grid0 = meteva.base.get_grid_of_data(grd_list[0])
    nmin = min(grid0.nlon,grid0.nlat)
    if delta > nmin/2:
        delta = int(nmin/2)

    nv = len(grd_list)



    rx = delta+ delta%step
    delta_y = np.arange(-rx,rx+1,step)
    delta_x = np.arange(-rx,rx+1,step)

    #自定义SI函数
    def SI(ntemp, delta):
        n1 = np.arange(ntemp)
        n2 = n1 + delta
        good = (n2 >= 0) & (n2 < ntemp)
        return np.vstack((n1[good], n2[good])).T

    vg = np.zeros((nv,len(delta_x),len(delta_y)))
    sample =np.zeros((nv,len(delta_x),len(delta_y)))
    dis  =np.zeros((nv,len(delta_x),len(delta_y)))
    for m in range(nv):
        dat = grd_list[m].values.squeeze()
        dat.flags.writeable = True
        dat[dat == meteva.base.IV] = np.nan
        for i in range(len(delta_x)):
            for j in range(len(delta_y)):
                MM = SI(ntemp=grid0.nlat, delta=delta_y[j])
                NN = SI(ntemp=grid0.nlon, delta=delta_x[i])

                i0 = MM[:, 0].astype(int)
                j0 = NN[:, 0].astype(int)
                ii, jj = np.meshgrid(i0, j0)
                dat_1 = dat[ii, jj].T
                i1 = MM[:, 1].astype(int)
                j1= NN[:, 1].astype(int)
                ii, jj = np.meshgrid(i1, j1)
                dat_2 = dat[ii, jj].T

                dis[m,i,j] = (delta_y[j]**2 + delta_x[i]**2)**0.5
                index = np.where(np.isnan(dat_1) | np.isnan(dat_1))
                sample[m,i,j]= dat_2.size - len(index[0])
                BigDiff = dat_1 - dat_2
                if q ==1:
                    vg[m,i,j] = np.nanmean(np.abs(BigDiff))
                else:
                    vg[m,i,j] = np.nanmean(BigDiff ** q)


    out = {"dis":dis,"vg":vg,  "sample":sample,"member":names,"delta_x":delta_x.tolist(),"delta_y":delta_y.tolist()}
    return out


def vgm_grd_mesh(grd_list, delta,step = 1, q = 1,
                  dpi = 300,show =False,save_path = None,sup_fontsize  = 10,ncol = 2,width = None,height = None,):

    out = structurogram_matrix(grd_list,delta=delta,step = step,q=q)
    mesh_dict = {"member":out["member"],"东西向平移网格数":out["delta_x"],"南北向平移网格数":out["delta_y"]}
    meteva.base.plot_tools.mesh(out["vg"],mesh_dict,axis_x="东西向平移网格数",axis_y="南北向平移网格数",width=width,
                                height=height,dpi = dpi,show=show,save_path=save_path,sup_fontsize=sup_fontsize,ncol=ncol)
    return out

def out_statistic(out,step):
    '''
    从变率样本矩阵中，统计出每一段距离范围内，平均的变率，以及变率值本身的方差
    :param dv_array:
    :param dmax:
    :param grade_count:
    :return:
    '''

    dmax = np.max(out["dis"][0])
    v_count = out["vg"].shape[0]
    grade_count = int(math.ceil(dmax/step))
    dmax = grade_count * step
    # c：count, m ：mean， v :var
    cmv_array = np.zeros((3, grade_count, v_count))


    for i in range(grade_count):
        index = np.where((out["dis"][0] > i * step) & (out["dis"][0] <= (i + 1) * step))
        for j in range(v_count):
            sample = out["sample"][j][index]
            vg = out["vg"][j][index]
            count = len(sample)
            if count > 0:
                cmv_array[0, i, j] = count
                cmv_array[1, i, j] = np.mean(vg)
                cmv_array[2, i, j] = np.var(vg)

    # 返回dmax，是方便后续程序确切了解组成采用dmax参数值
    return cmv_array, dmax


def plot_dv_statistic(cmv_array,dmax,member_list = None,width = None,height = None,xlabel = "距离（网格数）",ylabel = "变化幅度",
                      dpi = 300,show =False,save_path = None,sup_fontsize = 12):
    '''
    绘制变率统计结果
    :param cmv_array:
    :param dmax:
    :param member_list:
    :param width:
    :param height:
    :param xlabel:
    :param ylabel:
    :param dpi:
    :param show:
    :param save_path:
    :param sup_fontsize:
    :return:
    '''
    if height is None:
        height = 3

    grade_count = cmv_array.shape[-2]
    grade = dmax / grade_count

    grade_list = (grade * (np.arange(grade_count)+1)).astype(np.int16)
    grade_list_mid = (grade * (np.arange(grade_count)+0.5)).astype(np.int16)
    if width is None:
        width = meteva.base.tool.plot_tools.caculate_axis_width(grade_list,sup_fontsize*1.5,legend_num=1)
    fig = plt.figure(figsize=(width, height),dpi = dpi)

    v_count = cmv_array.shape[-1]
    label = []
    if member_list is None:

        label = ["观测"]
        for i in range(1,v_count):
            label.append('预报' + str(i))
    else:
        label.extend(member_list)

    color_list =meteva.base.color_tools.get_color_list(v_count)

    grid_plt = plt.GridSpec(5, 1, hspace=0)
    ax1 = plt.subplot(grid_plt[0:4, 0])
    plt.yticks(fontsize = sup_fontsize * 0.8)
    plt.ylabel(ylabel,fontsize = sup_fontsize * 0.9)
    max_list =[]
    for v in range(v_count):
        plt.plot(grade_list_mid,cmv_array[1,:,v],c = color_list[v],label= label[v])
        var = np.sqrt(cmv_array[2,:,v])
        plt.plot(grade_list_mid,cmv_array[1,:,v] - var,"--",c = color_list[v],linewidth =0.5)
        max_list.append(np.max(cmv_array[1,:,v] + var))
        plt.plot(grade_list_mid,cmv_array[1,:,v] + var,"--",c = color_list[v],linewidth =0.5)


    plt.xlim(0,grade_list[-1])
    maxv = np.max(np.array(max_list))
    plt.ylim(0,maxv*1.2)
    plt.legend(loc =2,ncol = 4,fontsize = sup_fontsize * 0.8)
    plt.setp(ax1.get_xticklabels(), visible=False)
    ax2 = plt.subplot(grid_plt[4, 0], sharex=ax1)
    plt.yticks(fontsize=sup_fontsize * 0.8)
    plt.ylabel("样本数",fontsize = sup_fontsize * 0.9)
    plt.xlim(0,grade_list[-1])
    plt.xlabel(xlabel,fontsize = sup_fontsize * 0.9)
    plt.bar(grade_list_mid, cmv_array[0,:,0], width=grade * 0.5,color = "grey")
    #plt.xlabel(xlabel=grade_list)

    plt.xticks(grade_list,fontsize = sup_fontsize * 0.8)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()


def vgm_grd(grd_list, delta,step = 1, q = 1,
                  dpi = 300,show =False,save_path = None,sup_fontsize  = 10,width = None,height = None):

    out = structurogram_matrix(grd_list,delta=delta,step=step,q=q)

    cmv_array,dmax = out_statistic(out,step=step)

    plot_dv_statistic(cmv_array,dmax,member_list=out["member"]
                      ,width=width,height = height,dpi = dpi,show=show,save_path =save_path,sup_fontsize=sup_fontsize)
    return cmv_array




    
if __name__ == "__main__":
    #dat = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\pert000.csv")

    filename_ob = r'H:\test_data\input\mem\mode\ob\rain03\20070111.000.nc'
    grd = meteva.base.read_griddata_from_nc(filename_ob)
    dat = grd.values.squeeze()
    look_SGM = vgm_grd_mesh(grd)    #读入的是dataframe数据，有行列名称
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    


__author__ = "The R & D Center for Weather Forecasting Technology in NMC, CMA"
__version__ = '1.8.3.2'
import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif']=["Source Han Sans CN","SimHei"] #用来正常显示中文标签
plt.rcParams['font.serif']=["Times New Roman"] #默认英文字体
plt.rcParams['axes.unicode_minus']=False #用来正常显示负号
from . import base
from . import method
from . import perspact
from . import product




# coding: utf-8
import numpy as np
import math
import meteva

def reset_max_min(vmax,vmin):
    dif = (vmax - vmin) / 10.0
    if dif ==0:
        inte = 1
    else:
        inte = math.pow(10, math.floor(math.log10(dif)))
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte
    if r < 3 and r >= 1.5:
        inte = inte * 2
    elif r < 4.5 and r >= 3:
        inte = inte * 4
    elif r < 5.5 and r >= 4.5:
        inte = inte * 5
    elif r < 7 and r >= 5.5:
        inte = inte * 6
    elif r >= 7:
        inte = inte * 8
    vmin = inte * ((int)(vmin / inte) - 1)
    vmax = inte * ((int)(vmax / inte) + 1)
    return vmax,vmin,inte

def sigmoid(inputX):
    return 1.0/(1+np.exp(-inputX))


def angle_of_line(ax,ay,bx,by):
    u = bx - ax
    v = by - ay
    s = math.sqrt(u * u + v * v)
    if s ==0:
        angle = 0
    else:
        if v >=0:
            angle = 90 - math.asin(u / s) * 180 / math.pi
        else:
            angle = 270 + math.asin(u / s) * 180 / math.pi

    return angle

def cross_angle(lines1,linee1, lines2,linee2):
    delta1 = [linee1[0] - lines1[0],linee1[1]-lines1[1]]
    delta2 = [linee2[0] - lines2[0],linee2[1]-lines2[1]]
    length1 = math.sqrt(math.pow(delta1[0],2) + math.pow(delta1[1],2))
    length2 = math.sqrt(math.pow(delta2[0],2) + math.pow(delta2[1],2))
    cos1 = (delta1[0] * delta2[0] + delta1[1] * delta2[1])/(length1 * length2)
    angle = math.acos(cos1) *180/ math.PI
    return angle


#经度纬度信息转换为直角坐标系
def lon_lat_to_cartesian(lon, lat, R=1):
    """
    calculates lon, lat coordinates of a point on a sphere with
    radius R
    """
    lon_r = np.radians(lon)
    lat_r = np.radians(lat)
    xyz = np.zeros((len(lon), 3))
    xyz[:, 0] = R * np.cos(lat_r) * np.cos(lon_r)
    xyz[:, 1] = R * np.cos(lat_r) * np.sin(lon_r)
    xyz[:, 2] = R * np.sin(lat_r)
    return xyz

def mean_iteration(count_old,mean_old,count_new,mean_new):
    count_total = count_new + count_old
    rate1 = count_old/count_total
    rate2 = count_new/count_total
    mean = rate1 * mean_old + rate2 * mean_new
    return mean

def ss_iteration(count_old,mean_old,ss_old,count_new,mean_new,ss_new):
    count_total = count_new + count_old
    rate1 = count_old/count_total
    rate2 = count_new/count_total
    mean_total = rate1 * mean_old + rate2 * mean_new
    #ss_total = ss_old * count_old
    #ss_total += count_old * math.pow((1- rate1) * mean_old - rate2 * mean_new,2)
    #ss_total += ss_new * count_new
    #ss_total += count_new * math.pow((1- rate2) * mean_new - rate1 * mean_old,2)
    #ss_total /= count_total
    ss_total= rate1*(ss_old + math.pow(rate2 *(mean_old - mean_new),2))
    ss_total += rate2 * (ss_new + math.pow(rate1 * (mean_old - mean_new), 2))

    return count_total,mean_total,ss_total

def sxy_iteration(count_old,meanx_old,meany_old,sxy_old,count_new,meanx_new,meany_new,sxy_new):
    import math
    count_total = count_new + count_old
    rate1 = count_old/count_total
    rate2 = count_new/count_total
    meanx_total = rate1 * meanx_old + rate2 * meanx_new
    meany_total = rate1 * meany_old + rate2 * meany_new
    sxy_total = sxy_old * count_old
    sxy_total += count_old * ((1- rate1) * meanx_old - rate2 * meanx_new) * ((1- rate1) * meany_old - rate2 * meany_new)
    sxy_total += sxy_new * count_new
    sxy_total += count_new * ((1- rate2) * meanx_new - rate1 * meanx_old) * ((1- rate2) * meany_new - rate1 * meany_old)
    sxy_total /= count_total
    return count_total, meanx_total, meany_total,sxy_total

def get_index(X,level_list):
    levels = np.array(level_list)
    levels.sort()
    index = np.zeros_like(X)
    index[...] = len(levels)
    for i in range(len(levels)):
        level = levels[i]
        index[X < level] = i - 1
    return index

def greatest_common_divisor(value_list):

    value_set = np.array(list(set(value_list))).astype(np.int32)

    # 获取最小值
    x = value_set[0]
    value_set1 = value_set[1:]
    for i in range(len(value_set1)):
        y = value_set1[i]
        if x > y:
            smaller = y
        else:
            smaller = x
        for i in range(1, smaller + 1):
            if ((x % i == 0) and (y % i == 0)):
                hcf = i
        x = hcf
    return x


def u_v_to_s_d(u,v):
    s = np.sqrt(np.power(u, 2) + np.power(v, 2))
    d = np.zeros(u.shape)
    index = np.where(s != 0)
    vd_n0 = v[index]
    ud_n0 = u[index]
    s_n0 = s[index]
    ag_n0 = 180 - np.arccos(vd_n0 / s_n0) * 180 / np.pi
    ag_n0[ud_n0 > 0] = 360 - ag_n0[ud_n0 > 0]
    d[index] = ag_n0
    s[u==meteva.base.IV] = meteva.base.IV
    d[u==meteva.base.IV] = meteva.base.IV
    return s,d



def tran_direction_to_8angle(direction):
    '''
    将0-360度的风向，转换成8个方位，分布用0,1,2，。。。7代表北风，东北风，东方，东南风，南风，西南风，西风，西北风
    :param direction: 风向角度， 0-360度。 任意维numpy数组
    :return: 风向方位，0-7，和direction的shape一致的numpy数组
    '''
    angles = np.zeros(direction.shape)
    for i in range(1,8):
        angles[np.abs(direction - i * 45) <= 22.5] = i
    return angles

def tran_speed_to_14grade(speeds):
    '''
    将风速（m/s)，转换成14个风速等级，分布用0,1,2，。。。13代表静风，1级、2级，。。。，12级和大于等于13级
    :param speeds: 风速（m/s)。 任意维numpy数组
    :return: 风速等级，和speeds的shape一致的numpy数组
    '''
    grades = np.zeros(speeds.shape)
    gs = [0,0.3,1.6,3.4,5.5,8.0,10.8,13.9,17.2,20.8,24.5,28.5,32.7,37,300]
    for i in range(1,14):
        gi = gs[i]
        grades[speeds>=gi] = i
    return grades

def s_d_to_u_v(speed,direction):
    u = -speed * np.sin(direction * 3.14 / 180)
    v = -speed * np.cos(direction * 3.14 / 180)
    return u,v



def ctl_proj(grid0,proj_para,dat):
    #投影中心点

    inter_i = meteva.base.ER * 1000 / proj_para["dx"]
    inter_j = meteva.base.ER * 1000 / proj_para["dy"]
    if proj_para["type"] == "lcc":
        mlon = proj_para["proj_mlon"]* math.pi/180

        #球面转扇形
        lats1 = (grid0.slat + np.arange(grid0.nlat) * grid0.dlat) * math.pi / 180
        lons1 = (grid0.slon + np.arange(grid0.nlon) * grid0.dlon) * math.pi / 180
        lons,lats = np.meshgrid(lons1,lats1)
        lons = lons.flatten()
        lats = lats.flatten()
        #根据lon 计算扇形fanR
        slat = proj_para["proj_slat"] * math.pi/180
        elat = proj_para["proj_elat"] * math.pi/180
        xs = math.cos(slat)
        ys = math.sin(slat)
        xe = math.cos(elat)
        ye = math.sin(elat)
        a = (ys - ye)/(xs - xe)  #在R方向投影线性系数
        b = (ye * xs - ys * xe)/(xs - xe)
        r = np.tan(lats)
        x_ar = b/(r-a)
        ac = math.sqrt(1 + a * a)
        fanR = x_ar * ac

        #计算角度比例
        dx_slat = math.cos(slat)
        fanR_slat = b/(math.tan(slat) - a) * ac
        angle_rate = dx_slat / fanR_slat


        dlons = (lons - mlon) * angle_rate

        fanX = fanR * np.sin(dlons)
        fanY = -fanR * np.cos(dlons)

        dmodel_lon = (proj_para["model_lon"] * math.pi/180 - mlon) * angle_rate
        model_lat = proj_para["model_lat"] * math.pi / 180
        model_fanR =b/(math.tan(model_lat)-a)* ac
        model_fanX = model_fanR * np.sin(dmodel_lon)
        model_fanY = -model_fanR * np.cos(dmodel_lon)

        fan_i = (fanX - model_fanX) * inter_i + proj_para["model_i"]
        fan_j = (fanY - model_fanY) * inter_j + proj_para["model_j"]
        nx_1 = proj_para["nx"] - 1
        ny_1 = proj_para["ny"] - 1
        in_grid_index = np.where((fan_i >= 0) & (fan_i < nx_1) & (fan_j >= 0) & (fan_j < ny_1))[0]
        fan_i_in = fan_i[in_grid_index]
        fan_j_in = fan_j[in_grid_index]
        ig = fan_i_in.astype(dtype = 'int16')
        jg = fan_j_in.astype(dtype = 'int16')
        dx = fan_i_in - ig
        dy = fan_j_in - jg
        c00 = (1 - dx) * (1 - dy)
        c01 = dx * (1 - dy)
        c10 = (1-dx) * dy
        c11 = dx * dy
        ig1 = ig + 1
        jg1 = jg + 1
        dat_sta = c00 * dat[jg, ig] + c01 * dat[jg, ig1] + c10 * dat[jg1, ig] + c11 * dat[jg1, ig1]
        data_re = np.zeros(grid0.nlat * grid0.nlon)
        data_re[in_grid_index] = dat_sta[:]
        grd = meteva.base.grid_data(grid0,data_re)
        return grd


def isRayIntersectsSegment(poi,s_poi,e_poi): #[x,y] [lng,lat]
    #输入：判断点，边起点，边终点，都是[lng,lat]格式数组
    if s_poi[1]==e_poi[1]: #排除与射线平行、重合，线段首尾端点重合的情况
        return False
    if s_poi[1]>poi[1] and e_poi[1]>poi[1]: #线段在射线上边
        return False
    if s_poi[1]<poi[1] and e_poi[1]<poi[1]: #线段在射线下边
        return False
    if s_poi[1]==poi[1] and e_poi[1]>poi[1]: #交点为下端点，对应spoint
        return False
    if e_poi[1]==poi[1] and s_poi[1]>poi[1]: #交点为下端点，对应epoint
        return False
    if s_poi[0]<poi[0] and e_poi[1]<poi[1]: #线段在射线左边
        return False

    xseg=e_poi[0]-(e_poi[0]-s_poi[0])*(e_poi[1]-poi[1])/(e_poi[1]-s_poi[1]) #求交
    if xseg<poi[0]: #交点在射线起点的左侧
        return False
    return True  #排除上述情况之后

def isPoiWithinPoly(poi,poly):
    #输入：点，多边形三维数组
    #poly=[[[x1,y1],[x2,y2],……,[xn,yn],[x1,y1]],[[w1,t1],……[wk,tk]]] 三维数组

    #可以先判断点是否在外包矩形内
    #if not isPoiWithinBox(poi,mbr=[[0,0],[180,90]]): return False
    #但算最小外包矩形本身需要循环边，会造成开销，本处略去
    sinsc=0 #交点个数
    for epoly in poly: #循环每条边的曲线->each polygon 是二维数组[[x1,y1],…[xn,yn]]
        np = len(epoly)
        for i in range(np-1): #[0,len-1]
            s_poi=epoly[i]
            e_poi=epoly[i+1]
            if isRayIntersectsSegment(poi,s_poi,e_poi):
                sinsc+=1 #有交点就加1
        if epoly[np-1][0] != epoly[0][0] or  epoly[np-1][1] != epoly[0][1]:
            s_poi = epoly[np-1]
            e_poi = epoly[0]
            if isRayIntersectsSegment(poi, s_poi, e_poi):
                sinsc += 1  # 有交点就加1

    return True if sinsc%2==1 else  False


def distance_in_a_straight_line(lon1,lat1,lon2,lat2):
    '''
    计算地球表面两个点之间的直线距离
    :param lon1:
    :param lat1:
    :param lon2:
    :param lat2:
    :return:
    '''
    xyz1 = lon_lat_to_cartesian(lon1, lat1, R=meteva.base.ER)
    xyz2 = lon_lat_to_cartesian(lon2,lat2,R = meteva.base.ER)
    dxyz = xyz2 - xyz1
    dis2 = np.sum(dxyz*dxyz,axis=1)
    dis1 = np.sqrt(dis2)
    return dis1


def distance_on_earth_surface(lon1,lat1,lon2,lat2):
    '''
    计算地球表明两个点之间的球面距离
    :param lon1:
    :param lat1:
    :param lon2:
    :param lat2:
    :return:
    '''

    a1 = lon1 * math.pi/180
    b1 = lat1 * math.pi/180
    a2 = lon2 * math.pi/180
    b2 = lat2 * math.pi/180

    dis = meteva.base.ER * np.arccos(np.cos(b1)*np.cos(b2)*np.cos(a1-a2) + np.sin(b1)*np.sin(b2))

    if isinstance(dis,np.ndarray):
        dis[np.isnan(dis)] =0
    else:
        if np.isnan(dis):
            dis = 0
    return dis



# -*- coding: utf-8 -*-
"""
Created on Sun Aug  2 16:55:54 2020

@author: 1
"""

#============================  deltammSqCen()  ==============================
import numpy as np
import pandas as pd
from . import censqdelta
from .centmatch import restructuring

#提取数据
def pick_labels(data):
    ret = {}
    for item, values in data.items():

        if isinstance(item,int):
            #print(item, values)
            ret0 = {item : values}
            ret.update(ret0)
    return ret

#以观测场为准，将预报场匹配到观测场
def bdFun(Id,nrow,ncol, OB, FC, p, const, N, show):
    #OB = X    #参数取值
    #FC = Xhat    #参数取值
    #一一对比计算预报场和观测场的delta结果
    #OB = pick_labels(OB)
    #FC = pick_labels(FC)
    res = np.array(())

    ob1 = np.zeros((nrow,ncol))
    fo1 = np.zeros((nrow,ncol))

    for j in range(1, len(OB) + 1):
        for k in range(1, len(FC) + 1):
            #print("观测目标：%d  ,"%j ,  "预报目标：%d"%k)
            ob1[:] = 0
            fo1[:] = 0
            ob1[OB[j]] = 1
            fo1[FC[k]] = 1
            res_0 = censqdelta.censqdelta(x = ob1,
                             y = fo1, N = N, const = const, p = p)
            res = np.append(res,res_0)
            if (show):
                print("Calculating Baddeley Delta Metric between forecast feature {0} and observed feature {1}\n".format(j, k))                
    return res

#以预报场为准，将观测场匹配到预报场
def foFun(Id,nrow,ncol, OB, FC, p, const, N, show):
    res = np.array(())

    ob1 = np.zeros((nrow,ncol))
    fo1 = np.zeros((nrow,ncol))

    for j in range(1, len(FC) + 1):
        for k in range(1, len(OB) + 1):
            #print("预报目标：%d  ,"%j ,  "观测目标：%d"%k)
            ob1[:] = 0
            fo1[:] = 0
            ob1[OB[k]] = 1
            fo1[FC[j]] = 1
            res_0 = censqdelta.censqdelta(x = fo1,
                             y = ob1, N = N, const = const, p = p)
            res = np.append(res,res_0)
            if (show):
                print("Calculating Baddeley Delta Metric between forecast feature {0} and observed feature {1}\n".format(j, k))                
    return res

#判断元素是否存在
def efun(x, ftr):
    for i in range(len(x)):
        if (x['%d'%i] in np.array(ftr)):
            x['%d'%i] = True
        else :
            x['%d'%i] = False
    return x

def mergeIdentifier(x):
    if any( np.shape(x)) == 0:
        return None
    matchlen = np.shape(x)[0]
    fuq = np.unique(x['Forecast'])
    ouq = np.unique(x['Observed'])
    flen = len(fuq)
    olen = len(ouq)
    out = []
    if matchlen == flen and matchlen > olen:
        for i in range(olen):
            if x[:, 'Observed'] == ouq[i]:
                out[i] = x
    elif matchlen > flen and matchlen == olen:
        for i in range (flen):
            if x['Forecast'] == fuq[i]:
                out[i] = x
    elif matchlen > flen and matchlen > olen:
        if matchlen > 1:
            idx = np.zeros(matchlen)
            idx[0] = 1
            for i in range(1, matchlen):
                idF = x.loc[0:i, 'Forecast'] == x.loc[i, 'Forecast']
                idO = x.loc[0:i, 'Observed'] == x.loc[i, 'Observed']
                if any(idF):
                    idx[i] = idx[0:i][0]
                elif any(idO):
                    idx[i] = idx[0:i][0]
                else:
                    idx[i] = max(idx[0:i]) + 1
        else:
            return (None)
        for j in range (0, max(idx).astype(int)):
            out = x[idx == j]
    elif matchlen == flen and matchlen == olen:
        return None
    return out 

def merge_or_label_index(nlat,nlon,label_index1,label_index2):
    label = np.zeros((nlat,nlon))
    label[label_index1] = 1
    label[label_index2] = 1
    return np.where(label == 1)

def deltammSqcen(x, p, max_delta, const, N, show):

    #a = look    #look为featureFinder生成的结果
    out = {}
    X = x['grd_ob_features']    #R:引用的为单独标记的连通域
    Xhat = x['grd_fo_features']    #R:引用的为单独标记的连通域
    x['ob'] = x['grd_ob']
    x['fo'] = x['grd_fo']
    grid0 = x["grid"]
    #单独提取被标记的对象
    X = pick_labels(X)
    Xhat = pick_labels(Xhat)
    xdim = np.shape(x['ob'])     #R:引用属性里面的维度(shape)。观测场和预报场的要素数量一致
    if (N == None):
        N = max(xdim)
        if (N % 2 == 0):
            N = N + 1
    #R:函数内部定义bdFun函数，python：在函数外部单独定义
    n = len(X)    
    m = len(Xhat)
    no_X = np.all(X) == None or n == 0
    no_Xhat = np.all(Xhat) == None or m == 0

    if (no_X or no_Xhat):
        if (no_X and no_Xhat and show):
            print("No identified features in either field.  Therefore, no matches/merges made.\n")
        elif (no_X and show):
            print("No identified observed features.  Therefore, no matches/merges made.\n")
        elif (no_Xhat and show):
            print("No identified model features.  Therefore, no matches/merges made.\n")
        funmatched = np.arange(1, m + 1, 1)    #python下标从0开始
        vxunmatched = np.arange(1, n + 1, 1)
        #matches = np.array(robjects.r('''
        #                     cbind(integer(0), integer(0))
        #                     '''))
        matches = np.array([])
        merges = None
        unmatched = {"X":vxunmatched, "Xhat":funmatched}
        out.update({"unmatched":unmatched, "matches":matches, "merges":merges, "class_":"matched"})
        return out 

    if (show):
        print("Step 1: Finding Upsilon matrix containing the Baddeley delta between each individual feature across fields.\n")
    #ind = np.array(robjects.r('''cbind(rep(1:n, m), rep(1:m, each = n))'''))    #传参有问题
    rep00_1 = np.arange(1, n + 1, 1)
    rep00 = np.tile(rep00_1, m)
    rep0_1 = np.arange(1, m + 1, 1)
    rep0 = rep0_1.repeat(n, axis = 0)    #按行进行元素重复
    ind = np.vstack((rep00, rep0)).T    #数组的形状未转置
    #第一步：一一对比观测场和预报场,观测场(列号)→预报场(行号)
    Upsilon = bdFun(Id = ind,nrow=grid0.nlat,ncol=grid0.nlon, OB = X, FC = Xhat, p = p, const = const, N = N, show = show)
    Upsilon = Upsilon.reshape(len(X), len(Xhat))
    #一一对比预报场→观测场
    Upsilon_fo = foFun(Id = ind,nrow=grid0.nlat,ncol=grid0.nlon, OB = X, FC = Xhat, p = p, const = const, N = N, show = show)
    Upsilon_fo = Upsilon_fo.reshape(len(Xhat), len(X))

    #对Upsilon二维数组排序
    o_Ksi = np.argsort(Upsilon, axis=1)    #按行排序,返回的是观测场的序号
    o_Psi = np.argsort(Upsilon, axis=0)    #按列排序,返回的是预报场的序号
    Ksi = np.sort(Upsilon, axis=1)    #按列排序,不改变数值
    Psi = np.sort(Upsilon, axis=0)    #按行排序,不改变数值

    if(show):
        print("Finding {0} potential forecast merges for each of the {1} observed features.\n".format(m - 1, n))
    
    #以观测场为准，合并预报场    
    for j in range (len(X)):
        #猜测：因为Ksi是对列调整，取值是观测场X；Psi是对行调整，取值是预报场Xhat
        #o <- (1:m)[o.Ksi[j, ]]
        o = o_Ksi[j, :]
        #print("预报场目标序号：", o)
        #外层循环观测场，控制合并的次数；内层循环取预报场的目标序号，控制合并的内容
        if (len(X) >= 2) and len(o)>1:
            #R:union.owin合并两个图像
            newobj = {1: merge_or_label_index(grid0.nlat,grid0.nlon,Xhat[o[0] + 1],Xhat[o[1] + 1])}
            #newobj = {'labels_%d'%1:Xhat['labels_%d'%(o[0] + 1)] | Xhat['labels_%d'%(o[1] + 1)] }
        else :
            newobj = Xhat
        if (len(X) >= 3) and len(o)>1:
            #for (i in 3:length(Xhat)) newobj[[i - 1]] <- union.owin(newobj[[i - 2]], Xhat[[o[i]]])
            for i in range(3, len(Xhat) + 1):    #取预报场的目标序号
                #print("预报场目标序号：", i)
                newobj_0 = {(i - 1): merge_or_label_index(grid0.nlat,grid0.nlon,newobj[i - 2] , Xhat[o[i - 1] + 1])}
                #newobj_0 = {'labels_%d'%(i - 1):newobj['labels_%d'%(i - 2)] | Xhat['labels_%d'%(o[i - 1] + 1)]}
                newobj.update(newobj_0)

    if (m > 1):
        if (show):
            print("Calculating delta metrics for forecast merges.\n")
        rep00_1 = np.arange(1, n + 1, 1)
        rep00 = np.tile(rep00_1, m -1)
        rep0_1 = np.arange(1, m, 1)
        rep0 = rep0_1.repeat(n, axis = 0)    #按行进行元素重复
        ind = np.vstack((rep00, rep0)).T 
        look = []
        for i in range(len(ind)):
            look_0 = str(ind[i, 0]) + '-' + str(ind[i, 1])
            look.append(look_0)
        #look = set(look)    #去掉重复的匹配
        tmp = bdFun(Id = ind,nrow=grid0.nlat,ncol=grid0.nlon, OB = X, FC = newobj, p = p, const = const, N = N, show = show)
        tmp = tmp.reshape(len(X), len(newobj))
        Ksi[:, 1:len(Xhat)] = tmp
    else:
        Ksi = Upsilon
    if (show):
        print("\nFinding {0} potential observed merges for each of the {1} forecast features.\n".format((n - 1), m))


    #以预报场为准，合并观测场    
    for k in range(len(Xhat)):
        if (show):
            print(k)
        #o <- (1:n)[o.Psi[, k]]
        o = o_Psi[:, k] 
        #print("观测场目标序号：", o)
        if (n >= 2):
            #newobj = {'labels_%d'%1:X['labels_%d'%(o[0] + 1)] | X['labels_%d'%(o[1] + 1)]}
            newobj = {1: merge_or_label_index(grid0.nlat,grid0.nlon,X[o[0] + 1] , X[o[1] + 1])}
        else:
            newobj = X
        if (n >= 3):
            for i in range(2, n):
                #newobj_0 = {'labels_%d'%(i):newobj['labels_%d'%(i - 1)] | X['labels_%d'%(o[i - 1] + 1)]}
                newobj_0 = {i: merge_or_label_index(grid0.nlat,grid0.nlon,newobj[i - 1] , X[o[i - 1] + 1])}
                newobj.update(newobj_0)
    if (n > 1):
        if (show):
            print("\nAll metrics for forecast merges found.  Calculating delta metrics for observed merges.\n")
        rep00_1 = np.arange(1, n , 1)
        rep00 = np.tile(rep00_1, m)
        rep0_1 = np.arange(1, m + 1, 1)
        rep0 = rep0_1.repeat(n - 1, axis = 0)    #按行进行元素重复
        ind = np.vstack((rep00, rep0)).T 
        look = []
        for i in range(len(ind)):
            look_0 = str(ind[i, 0]) + '-' + str(ind[i, 1])
            look.append(look_0)
        #look = set(look)    #去掉重复的匹配
        tmp = bdFun(Id = ind,nrow=grid0.nlat,ncol=grid0.nlon, OB = newobj, FC = Xhat, p = p, const = const, N = N, show = show)
        tmp = tmp.reshape(len(newobj), len(Xhat))
        Psi[1:, 0:m] = tmp
    else:
        Psi  = Upsilon
    bigQ = np.zeros((3, n, m))    #三维数组:m*n,与Upsilon/Ksi/Psi的shape一致
    bigQ[0, :, :] = Upsilon
    bigQ[1, :, :] = Psi
    bigQ[2, :, :] = Ksi
    out.update({'Q':bigQ})
    if (np.all(bigQ > max_delta)):
        if (show):
            print("\nAll delta metrics are larger than max.delta, no merges/matches.\n")
        funmatched = np.arange(m)
        vxunmatched = np.arange(n)
        unmatched = {'ob':vxunmatched, 'fo':funmatched}
        matches = np.array([])
        out.update({'unmatched':unmatched, 'matches':matches, 'merges':"NULL"})
        return out
    else:
        if (show):
            print("Psi:\n")
            print(Psi)
            print("Ksi:\n")
            print(Ksi)
            print("\nAll Baddeley metrics found.  Book keeping ...\n")

        
        '''
        J = {}
        K = {}
        j_0 = []
        k_0 = []
        for jk in range( 3 * n * m):
            if (jk < n*m):
                j_0 = ind[jk, 0]
                k_0 = ind[jk, 1]
                J.update({'%d'%(jk): j_0})
                K.update({'%d'%(jk): k_0})
            elif ((jk > n * m) and (jk <= 2 * n * m)):
                k_0 = ind[jk, 1]
                jj = np.arange(ind[jk, 0])
                J.update({'%d'%(jk): jj})
                j_0 = o_Psi[:n, ind[jk, 1] - 1]
                J.update({'%d'%(jk): j_0})
            else :
                j_0 = ind[jk, 0]
                kk = np.arange(ind[jk, 1])
                J.update({'%d'%(jk): kk})
                k_0 = o_Ksi[ind[jk, 0] - 1, :m]
                K.update({'%d'%(jk): k_0})
                
        Iter = 1
        matches = np.zeros((1,2))
        nn = np.arange(n)
        mm = np.arange(m)
        bigQ[bigQ > max_delta] = None
        bigQ_arr = bigQ.reshape(bigQ.size, ).copy()    #将三维数组展开成一维
        
        while (len(nn) > 0 and len(mm) > 0 and Iter < 3 * n * m + 1):
            if ((bigQ != None).any()):    #R: if (any(!is.na(bigQ)))
                #R:bigQ存为list形式
                #bigQ_list = [item for sublist in bigQ for item in sublist]
                #bigQ_list = [item for sublist in bigQ_list for item in sublist]
                #minQ = min(bigQ_arr)    #获取bigQ_list列表中的最小值
                Id = np.argmin(bigQ_arr)    #获取bigQ_list列表中的最小值的索引
                #Id = ind[minQ_index -1, 3]    #ind是调用R生成的，是从1开始
                if Id != None :
                    Id = Id
                #Id = Id[Id != None]
                #R: Id = Id[1]
                vx = J[str(Id)]
                fc = K[str(Id)]
                newmatch = np.vstack((vx, fc)).T
                matches = np.vstack((matches, newmatch))
                bigQ_arr[int(Id)] = None
                Id1 = efun(x = J.copy(), ftr = vx)
                Id2 = efun(x = K.copy(), ftr = fc)
                Id0 = np.array(list(Id1.values()))|np.array(list(Id2.values()))
                bigQ_arr[np.argwhere(Id0 == True)] = None
                vx = np.array(J[str(Id)])
                fc = np.array(K[str(Id)])
                nn[nn not in vx] = nn
                mm[mm not in fc] = fc
                Iter += 1
            else :
                Iter = float('inf')
        '''
        #out$unmatched <- list(X = nn, Xhat = mm)
        #colnames(matches) <- c("Forecast", "Observed")
        #out$matches <- matches
        #merges <- MergeIdentifier(matches)
        #out$merges <- merges
        
        #以观测场为准，匹配结果为：
        X_number = np.arange(len(X))
        #print(o_Ksi.shape)
        #print(o_Ksi)
        Xhat_number = o_Ksi[:, 0]
        Xmatches_number = pd.DataFrame(np.array((X_number+1, Xhat_number+1)).T, columns = ['Ob', 'Fcst'])
        #print(Xmatches_number)
        
        #(矩阵排列暂时存在问题)以预报场为准，匹配结果为：
        Xhat_number = np.arange(len(Xhat))
        X_number = o_Ksi[0, :]
        Xhatmatches_number = pd.DataFrame(np.array((Xhat_number+1, X_number+1)).T, columns = ['Fcst', 'Ob'])
        
        #调整unmatched存放结构
        a_obs = set(Xmatches_number['Ob'])
        a_fcst = set(Xmatches_number['Fcst'])
        if len(a_fcst) != len(Xhat):
            unmatched_X = set(np.arange(1,m+1)) - set(a_fcst)    #X未匹配的目标
            #print(unmatched_X)    
        else :
            unmatched_X = 'NULL'

        if len(a_obs) != len(X) :
            unmatched_Xhat = set(np.arange(1,n+1)) - set(a_obs)    #Xhat未匹配的目标
            #print(unmatched_Xhat)    
        else :
            unmatched_Xhat = 'NULL'
        unmatched = {'ob':unmatched_Xhat, 'fo':unmatched_X}


        matches = np.array([list(Xmatches_number['Fcst']), list(Xmatches_number['Ob'])])

        #unmatched = pd.DataFrame(unmatched, columns = ["Forecast", "Observed"])
        merges = restructuring(list(Xmatches_number['Fcst']), list(Xmatches_number['Ob']))
        out.update({'unmatched':unmatched, 'matches':matches, 'merges':merges, 'Xmatches_number':Xmatches_number,
                    'Xhatmatches_number':Xhatmatches_number})
        out.update({'identifier_function':"convthresh", 'identifier_label':"Convolution Threshold"})
            
    return out
    
#=============================  Example  ===================================
'''
#makeSpatialVx参数
hold = tra_makeSpatialVx_test.makeSpatialVx(X = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\pert000.csv"), \
                    Xhat = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\pert004.csv"), \
                    loc = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\ICPg240Locs.csv"), \
                    thresholds = [0.01, 20.01], projection = True, subset = None, timevals = None, reggrid = True,\
                    Map = True, locbyrow = True, fieldtype = "Precipitation", units = ("mm/h"), dataname = "ICP Perturbed Cases", obsname = "pert000", \
                    modelname = "pert004" , q = (0, 0.1, 0.25, 0.33, 0.5, 0.66, 0.75, 0.9, 0.95), qs = None)
#FeatureFinder参数
look = FeatureFinder_test_PA3.featureFinder(Object = hold, smoothfun = "disk2dsmooth", \
                     dosmooth = True, smoothpar = 17, smoothfunargs = None,\
                     thresh = 310, idfun = "disjointer", minsize = np.array([1]),\
                     maxsize = float("Inf"), fac = 1, zerodown = False, timepoint = 1,\
                     obs = 1, model = 1)


#deltaSqcen参数
x = lookFeatureFinder.copy()
p = 2
max_delta = float("Inf")
const = float("Inf")
N = 701    #设定矩形大小,需要设定比行列数大至少100，否则坐标经过变化以后会超出范围，程序报错
show = False    #选择是否需要在屏幕打印

lookdeltammSqcen = deltammSqcen(x = look.copy(), p = 2, max_delta = float("Inf"), const = float("Inf"), N = 701, show = False)
'''










#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 12 00:05:09 2022

@author: tangbuxing
"""

import numpy as np
import pandas as pd
import seaborn as sns
from matplotlib import pyplot
from sklearn.utils import resample
from matplotlib import pyplot as plt

# from statsmodels.base.model import GenericLikelihoodModelResults


# test_sample = np.array([1.865, 3.053, 1.401, 0.569, 4.132])
# r的结果
res_r = pd.read_csv(r"/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/booted003.csv")
# python重采样
test_sample = pd.read_csv(r"/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/booted001data.csv")
# boots_samples_01 = [np.mean(test_sample.sample(19, replace=True)) for _ in range(100000)]
boots_samples = [np.mean(resample(test_sample, n_samples=361)) for _ in range(10000)]  # n_samples是受tseries控制

# boots_samples = [np.mean(resample(test_sample, n_samples = 19)) for _ in range(500)]

# boot_var = [np.var(resample(test_sample)) for _ in range(19)]

# 画图
# a_r=pd.cut(res_r['t'],[-0.6,-0.5,-0.4,-0.3,-0.2,-0.1,0,0.1,0.2,0.3,0.4,0.5,0.6])
a_r = pd.cut(res_r['t'], [-1.0, -0.9, -0.8, -0.7, -0.6, -0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6])

b_r = a_r.value_counts()
b_r2 = b_r.sort_index()

# 画柱状图
c = {'section': b_r2.index, 'frequency': b_r2.values}
e = pd.DataFrame(c)
plt.rcParams['font.sans-serif'] = ['SimHei']
ax = plt.figure(figsize=(10, 5)).add_subplot(111)
sns.barplot(x="section", y="frequency", data=e, palette="Set3")  # palette设置颜色

aa = np.array(boots_samples).reshape(10000, )
a_p = pd.cut(aa, [-1.0, -0.9, -0.8, -0.7, -0.6, -0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6])
b_p = a_p.value_counts()
b_p2 = b_p.sort_index()
c = {'section': b_p2.index, 'frequency': b_p2.values}
e = pd.DataFrame(c)
plt.rcParams['font.sans-serif'] = ['SimHei']
ax = plt.figure(figsize=(10, 5)).add_subplot(111)
sns.barplot(x="section", y="frequency", data=e, palette="Set3")  # palette设置颜色

names = range(8, 21)
names = [str(x) for x in list(names)]

x = range(len(names))
y_train = [-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6]
y_test = [0.838, 0.840, 0.840, 0.834, 0.828, 0.814, 0.812, 0.822, 0.818, 0.815, 0.807, 0.801, 0.796]

plt.plot(y_train, b_p2, marker='o', mec='r', mfc='w', label='python')
plt.plot(y_train, b_r2, marker='*', ms=10, label='r')
plt.legend()  # 让图例生效
plt.xticks(y_train, names, rotation=1)

res_r = pd.read_csv(r"/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/rnorm00.csv")
a_r = pd.cut(res_r['x'], [-3.0, -2.5, -2.0, -1.5, -1.0, -0.5, 0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0])

b_r = a_r.value_counts()
b_r2 = b_r.sort_index()
c = {'section': b_r2.index, 'frequency': b_r2.values}
e = pd.DataFrame(c)
plt.rcParams['font.sans-serif'] = ['SimHei']
ax = plt.figure(figsize=(10, 5)).add_subplot(111)
sns.barplot(x="section", y="frequency", data=e, palette="Set3")  # palette设置颜色

mu, sigma = 0, 1  # mean and standard deviation
# z = np.random.normal(mu, sigma, 361)    #正态分布存在差异
z = np.random.randn(361)  # 均匀分布
aa = np.array(z).reshape(361, )
a_p = pd.cut(aa, [-3.5, -3.0, -2.5, -2.0, -1.5, -1.0, -0.5, 0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5])
b_p = a_p.value_counts()
b_p2 = b_p.sort_index()
c = {'section': b_p2.index, 'frequency': b_p2.values}
e = pd.DataFrame(c)
plt.rcParams['font.sans-serif'] = ['SimHei']
ax = plt.figure(figsize=(10, 5)).add_subplot(111)
sns.barplot(x="section", y="frequency", data=e, palette="Set3")  # palette设置颜色



#!/usr/bin/env python
# coding=utf8
from .httpclient import get_http_result

class GDSDataService:
    def __init__(self, gdsIp, gdsPort):
        self.gdsIp = gdsIp
        self.gdsPort = gdsPort #GDS服务器地址

    def getLatestDataName(self, directory, filter):
        return get_http_result(self.gdsIp, self.gdsPort, "/DataService" + self.get_concate_url("getLatestDataName", directory, "", filter))

    def getFileList(self,directory):
        return get_http_result(self.gdsIp,self.gdsPort,"/DataService" + self.get_concate_url("getFileList", directory, "",""))

    def getData(self, directory, fileName):
        return get_http_result(self.gdsIp, self.gdsPort, "/DataService" + self.get_concate_url("getData", directory, fileName, ""))


    # 将请求参数拼接到url
    def get_concate_url(self, requestType, directory, fileName, filter) :
        url = ""
        url += "?requestType=" + requestType
        url += "&directory=" + directory
        url += "&fileName=" + fileName
        url += "&filter=" + filter
        return url



# -*-coding:utf-8-*-
from .distmap import *


def intersect(a, b):
    a_x_min = a["xrange"][0]
    a_x_max = a["xrange"][1]
    a_y_min = a["yrange"][0]
    a_y_max = a["yrange"][1]
    b_x_min = b["xrange"][0]
    b_x_max = b["xrange"][1]
    b_y_min = b["yrange"][0]
    b_y_max = b["yrange"][1]
    x_min = max(a_x_min, b_x_min)
    x_max = min(a_x_max, b_x_max)
    y_min = max(a_y_min, b_y_min)
    y_max = min(a_y_max, b_y_max)
    x_range = [x_min, x_max]
    y_range = [y_min, y_max]
    window = {"x_range": x_range, "y_range": y_range}
    a = rebound(a, window)
    b = rebound(b, window)
    result = a
    am = np.array(a["m"])
    bm = np.array(b["m"])
    rlen, clen = am.shape
    m = am * bm
    result["m"] = m
    return result
